package org.enhydra.barracuda.contrib.dbroggisch.repopulation;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.forms.validators.*;
import org.enhydra.barracuda.core.comp.model.*;
import org.apache.log4j.*;
import java.util.*;

/**
 * @author Diez B. Roggisch <diez.roggisch@artnology.com>
 * This class eases the population of errors resulting from form-validation.
 * This is done by catching raised exceptions and if their sources are
 * FormElements, a mapping is made which consists of the static ERROR_PREFIX
 * and the FormElements name to a BComponent.<p>
 * If the Exception has set a text, the mapped component is a BText-component
 * with that text. Otherwise it's just a BComponent.<p>
 * When using this class, you should do two things:<p>
 * <ol>
 * <li>Subclass it and implement the TemplateModel-interface</li>
 * <li>Add ERROR_PREFIX + Element-names directives to your HTML-code, e.g <b>Dir::Get_Data.ModelName.e_Password</b></li>
 * </ol>
 * When processing the BTemplate, these directives will return null for 
 * elements without errors, resulting in discarding the nodes.
 * Or they return the associated components so the nodes stay, and text will be substituted.
 */
public class ErrorFormMap extends DefaultFormMap
{
    private Map errorMap = new HashMap();
    public static String ERROR_PREFIX = "e_";
    protected static Logger logger = Logger.getLogger(ErrorFormMap.class.getName());


    public FormMap validate(boolean deferExceptions) throws ValidationException {
				errorMap.clear();
				try {
						super.validate(deferExceptions);
				}
				catch(ValidationException ex) {
						if(logger.isDebugEnabled()) {
								logger.debug("Catched ValidationException");
						}
						List e_list;
						if(deferExceptions) {
								e_list = ex.getExceptionList();
						}
						else {
								e_list = new Vector();
								e_list.add(ex);
						}
	    
						for(int i = 0; i < e_list.size(); i++) {
								if(logger.isDebugEnabled()) {
										logger.debug("Visiting Exception " + i);
								}
								ValidationException vex = (ValidationException)e_list.get(i);
								if(vex.getSource() instanceof FormElement) {
										FormElement fel = (FormElement)vex.getSource();
										BComponent comp;

										logger.debug("Element name: " + fel.getName());
										if(vex.getMessage() != null && !vex.getMessage().equals("")) {
												if(logger.isDebugEnabled()) {
														logger.debug("Adding BText with " + vex.getMessage());
												}
												comp = new BText();
												((BText)comp).setText(vex.getMessage());
										}
										else {
												if(logger.isDebugEnabled()) {
														logger.debug("Adding BComponent");
												}
												comp = new BComponent();
										}
										errorMap.put(ERROR_PREFIX + fel.getKey(), comp);				
								}
						}
						throw ex;
				}
				return this;
    }
		

    /** If you want errors from whole-form-validation to be populated, simply use this as a 
     * convenient method to add mappings.
     */
    public void addErrorComponent(String key, BComponent comp) {
				errorMap.put(key, comp);
    }

    /** In your subclasses TemplateModel-interface call this.
     */
    public Object getErrorComponent(String key)
    {
				if(errorMap.containsKey(key)){
						return errorMap.get(key);
				}
				return null;
    }
}
