/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: AbstractTableModel.java,v 1.5 2002/06/11 01:19:02 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;
import org.enhydra.barracuda.core.comp.model.*;

/**
 * This class provides the abstract implementation
 * for a Table Model.
 *
 * <p>Note: this interface is designed to be used to ways. You can
 * either implement to return a specific number of rows/cols (in
 * which case hasMoreXXX() methods should return false) OR you 
 * can implement using the hasMoreXXX() methods (in which case the
 * getColumnCount(), getRowCount() methods should return -1.
 *
 * <p>The getXXXCount() method is more like the Swing JTable interface;
 * the hasMoreXXX() method makes it easier to implement tables where
 * you don't know the total number of records when you start.
 *
 * <p>Classes implementing this interface should generally support both 
 * methods: in other words, for every row/column, invoke the getItemAt()
 * method, then while the model has more rows/columns, it should again 
 * invoke getItemAt().
 */
public abstract class AbstractTableModel implements TableModel {

    protected ViewContext viewContext = null;
    protected List listeners = new ArrayList();

    //--------------- AbstractTemplateModel ----------------------
    /**
     * Add a listener to the template that's notified each time a change
     * to the data model occurs.
     *
     * @param ml the TemplateModelListener
     */
    public void addModelListener(ModelListener ml) {
        listeners.add(ml);
    }

    /**
     * Remove a listener
     *
     * @param ml the TemplateModelListener
     */
    public void removeModelListener(ModelListener ml) {
        listeners.remove(ml);
    }
    
    /**
     * Forwards the given notification event to all
     * <code>TemplateModelListeners</code> that registered
     * themselves as listeners for this template model.
     */
    public void fireModelChanged() {
        Iterator it = listeners.iterator();
        ModelListener ml = null;
        while (it.hasNext()) {
            ml = (ModelListener) it.next();
            ml.modelChanged(this);
        }
    }
//    protected List listeners = new ArrayList();

    //--------------- AbstractTableModel -------------------------
    /**
     * Add a listener to the table that's notified each time a change
     * to the data model occurs.
     *
     * @param l the TableModelListener
     */  
//    public void addTableModelListener(TableModelListener l) {
//        listeners.add(l);
//    }

    /**
     * Remove a listener from the table.
     *
     * @param l the ListDataListener
     */  
//    public void removeTableModelListener(TableModelListener l) {
//        listeners.remove(l);
//    }

    /**
     * Reset the model to its initial (unprocessed) state. This 
     * is a convenience method that gets invoked prior to the 
     * entire model being rendered. You only need to override this 
     * method if you want to do something (like reset internal counters)
     * before the model is queried
     */
    public void resetModel() {
        //nop    
    }

    /**
     * Notifies all listeners that all cell values in the table's
     * rows may have changed. The number of rows may also have changed
     * and the <code>JTable</code> should redraw the
     * table from scratch. The structure of the table (as in the order of the
     * columns) is assumed to be the same.
     *
     * @see TableModelEvent
     */
//    public void fireTableDataChanged() {
//        fireTableChanged(new TableModelEvent(this));
//    }

    /**
     * Notifies all listeners that the table's structure has changed.
     * The number of columns in the table, and the names and types of
     * the new columns may be different from the previous state.
     * If the <code>JTable</code> receives this event and its
     * <code>autoCreateColumnsFromModel</code>
     * flag is set it discards any table columns that it had and reallocates
     * default columns in the order they appear in the model. This is the
     * same as calling <code>setModel(TableModel)</code> on the
     * <code>JTable</code>.
     *
     * @see TableModelEvent
     */
//    public void fireTableStructureChanged() {
//        fireTableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
//    }

    /**
     * Notifies all listeners that rows in the range
     * <code>[firstRow, lastRow]</code>, inclusive, have been inserted.
     *
     * @param  firstRow  the first row
     * @param  lastRow   the last row
     * @see TableModelEvent
     */
//    public void fireTableRowsInserted(int firstRow, int lastRow) {
//        fireTableChanged(new TableModelEvent(this, firstRow, lastRow, TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
//    }

    /**
     * Notifies all listeners that rows in the range
     * <code>[firstRow, lastRow]</code>, inclusive, have been updated.
     *
     * @param firstRow  the first row
     * @param lastRow   the last row
     * @see TableModelEvent
     */
//    public void fireTableRowsUpdated(int firstRow, int lastRow) {
//        fireTableChanged(new TableModelEvent(this, firstRow, lastRow, TableModelEvent.ALL_COLUMNS, TableModelEvent.UPDATE));
//    }

    /**
     * Notifies all listeners that rows in the range
     * <code>[firstRow, lastRow]</code>, inclusive, have been deleted.
     *
     * @param firstRow  the first row
     * @param lastRow   the last row
     * @see TableModelEvent
     */
//    public void fireTableRowsDeleted(int firstRow, int lastRow) {
//        fireTableChanged(new TableModelEvent(this, firstRow, lastRow, TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));
//    }

    /**
     * Notifies all listeners that the value of the cell at 
     * <code>[row, column]</code> has been updated.
     *
     * @param row  row of cell which has been updated
     * @param column  column of cell which has been updated
     * @see TableModelEvent
     */
//    public void fireTableCellUpdated(int row, int column) {
//        fireTableChanged(new TableModelEvent(this, row, row, column));
//    }

    /**
     * Forwards the given notification event to all
     * <code>TableModelListeners</code> that registered
     * themselves as listeners for this table model.
     *
     * @param e  the event to be forwarded
     *
     * @see #addTableModelListener
     * @see TableModelEvent
     */
//    public void fireTableChanged(TableModelEvent e) {
//        Iterator it = listeners.iterator();
//        TableModelListener tml = null;
//        while (it.hasNext()) {
//            tml = (TableModelListener) it.next();
//            tml.tableChanged(e);
//        }
//    }

    /**
     * Return all the listeners for this given list model (returns 
     * a copy of the underlying List)
     *
     * @returns all the listeners for this given list model
     */
//    public List getListeners() { 
//        return new ArrayList(listeners); 
//    }

    //--------------- Contextual ---------------------------------
    /**
     * Specify the ViewContext. This method will generally be called
     * by the class that is using the model to actually render the data
     * in a view. The context will be specified prior to a render pass,
     * and the context will be reset to null after the render pass. 
     *
     * @param vc the current ViewContext
     */
    public void setViewContext(ViewContext ivc) {
        viewContext = ivc;
    }

    /**
     * Get the current ViewContext
     *
     * @return the current ViewContext
     */
    public ViewContext getViewContext() {
        return viewContext;
    }
}