/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ComponentGateway.java,v 1.11 2002/06/12 21:09:15 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp.helper;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.util.http.*;
import org.apache.log4j.*;

/**
 * <p>The component gateway is a servlet that provides a very simple
 * interface to the Barracuda component model
 */
public abstract class ComponentGateway extends HttpServlet {

    //public vars...eventually, these should probably be final
    protected static Logger logger = Logger.getLogger(ComponentGateway.class.getName());
    public static boolean printPretty = false;
    public static boolean preventCaching = false;
    public boolean recycleChildren = false;

    //-------------------- ComponentGateway ----------------------
    /**
     * <p>Handle the default HttpRequest. This is the method developers
     * will typically override. The developers shaould add any components
     * to the root component and then return the underlying DOM Document (that 
     * backs their components) so it can be rendered
     *
     * @param root the root component which will get rendered as a result 
     *         of this request
     * @param vc the ViewContext object describes what features the 
     *         client view is capable of supporting
     * @param req the servlet request
     * @param req the servlet response
     * @return the Document to be rendered
     * @throws ServletException
     * @throws IOException
     */
    public abstract Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException, RenderException, EventException;

    /**
     * <p>Handle an EventException. Basically, this is where we handle the 
     * really bad, unexpected type of event exceptions. Generally, as you code,
     * if you want to interrupt the dispatch and fire a new event, you should
     * throw an InterruptDispatchException. Only throw EventExceptions in
     * truly exceptional circumstances.
     *
     * @param vc the ViewContext object describes what features the 
     *         client view is capable of supporting
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleEventException (EventException e, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //first see what the base exception is...if it's a servlet exception
        //or an IOException, rethrow it...
        Exception rootException = NestableException.getRootException(e);
        if (rootException instanceof ServletException) throw (ServletException) rootException;
        if (rootException instanceof IOException) throw (IOException) rootException;

        //csc_061202.1 - added    
        //set the caching hdrs (this will allow the static resources to be cached by the browser)
        resp.setHeader("Cache-Control","max-age=0");
        resp.setDateHeader("Last-Modified", System.currentTimeMillis());

        //if we get an EventException, log it and print the base exception
        resp.setContentType("text/html");
        PrintWriter out = resp.getWriter();
        ExceptionUtil.logExceptionAsHTML(out, e, req);
        logger.warn("Unexpected event exception: ", e);
        if (rootException!=e) {
            logger.warn("Root Exception: ", rootException);
        }
    }

    /**
     * <p>Handle a RenderException. Basically, this is where we handle the 
     * really bad, unexpected type of errors that occur while unexpectedldy 
     * rendering the component hierarchy.
     *
     * @param vc the ViewContext object describes what features the 
     *         client view is capable of supporting
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleRenderException (RenderException e, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //first see what the base exception is...if it's a servlet exception
        //or an IOException, rethrow it...
        Exception rootException = NestableException.getRootException(e);
        if (rootException instanceof ServletException) throw (ServletException) rootException;
        if (rootException instanceof IOException) throw (IOException) rootException;

        //csc_061202.1 - added    
        //set the caching hdrs (this will allow the static resources to be cached by the browser)
        resp.setHeader("Cache-Control","max-age=0");
        resp.setDateHeader("Last-Modified", System.currentTimeMillis());

        //if we get a RenderException, log it and print the base exception
        resp.setContentType("text/html");
        PrintWriter out = resp.getWriter();
        ExceptionUtil.logExceptionAsHTML(out, e, req);
        logger.warn("Unexpected event exception: ", e);
        if (rootException!=e) {
            logger.warn("Root Exception:", rootException);
        }
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        return new DefaultDOMWriter(printPretty, preventCaching);
    }




    //-------------------- HTTPServlet ---------------------------
    /**
     * <p>By default the GET request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
//csc_102201.1_start
//        handleDefault(new HttpServletRequestWrapper(req), resp);
        boolean handled = ScriptDetector.checkClientReq(req, resp);
        if (!handled) handleDefault(new org.enhydra.barracuda.core.helper.servlet.HttpServletRequestWrapper(req), resp);
//csc_102201.1_end
    }
    
    /**
     * <p>By default the POST request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(new org.enhydra.barracuda.core.helper.servlet.HttpServletRequestWrapper(req), resp);
    }
    
    /**
     * <p>By default the OPTIONS request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doOptions(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(new org.enhydra.barracuda.core.helper.servlet.HttpServletRequestWrapper(req), resp);
    }
    
    /**
     * <p>By default the DELETE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(new org.enhydra.barracuda.core.helper.servlet.HttpServletRequestWrapper(req), resp);
    }
    
    /**
     * <p>By default the PUT request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPut(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(new org.enhydra.barracuda.core.helper.servlet.HttpServletRequestWrapper(req), resp);
    }
    
    /**
     * <p>By default the TRACE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doTrace(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {  
        handleDefault(new org.enhydra.barracuda.core.helper.servlet.HttpServletRequestWrapper(req), resp);
    }

    protected void handleDefault(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {

        long bmillis = 0;
        long smillis = 0;
        long emillis = 0;
        long elapsed1 = 0;
        long elapsed2 = 0;
        if (logger.isInfoEnabled()) bmillis = System.currentTimeMillis();
        if (logger.isInfoEnabled()) logger.info("Handling incoming HTTP request in "+this);
        
        ViewContext vc = null;
        try {
            //Session ObjectRepository
            ObjectRepository.setupSessionRepository(req);                                  //csc_022101.1
            
            //start by figuring out the ViewCapabilities
            if (logger.isDebugEnabled()) logger.debug("Create the ViewContext");
//csc_101701.1            vc = new DefaultViewContext(new ViewCapabilities(req, resp));
            vc = new DefaultViewContext(new ViewCapabilities(req, resp), req, resp);    //csc_101701.1
        
            //create our root component
            if (logger.isDebugEnabled()) logger.debug("Create component root");
            BComponent broot = new BComponent();
            broot.setName("Root");

            //give the implementation a chance to add any components to the root
            if (logger.isDebugEnabled()) logger.debug("Handling default");
            Document doc = handleDefault(broot, vc, req, resp);
            
            //now init the component
            if (logger.isDebugEnabled()) logger.debug("Invoking initCycle on component hierarchy");
            broot.initCycle();
            
            //now render the component
            if (logger.isDebugEnabled()) logger.debug("Rendering component hierarchy");
            if (logger.isDebugEnabled()) smillis = System.currentTimeMillis();
            broot.render(vc);
            if (logger.isDebugEnabled()) elapsed1 = System.currentTimeMillis()-smillis;

            //its possible the implementor may want to recycle children...if so, remove them
            //prior to calling destroy on the root
            if (recycleChildren) {
                if (logger.isDebugEnabled()) logger.debug("Recycling child components");
                List children = broot.getChildren();
                if (children!=null) {
                    for (int i=children.size()-1; i>=0; i--) {
                        broot.removeChild(i);            
                    }
                }
            }

            //now destroy the component
            if (logger.isDebugEnabled()) logger.debug("Invoking destroyCycle on component hierarchy");
            broot.destroyCycle();

            //csc_102201.1            
            //now adjust the outgoing page (this is critical to make sure we
            //can accurately detect client scripting)
            ScriptDetector.prepareClientResp(doc, vc);

            //now render the page
            if (logger.isDebugEnabled()) logger.debug("Rendering the DOM");
            if (logger.isInfoEnabled()) smillis = System.currentTimeMillis();
            this.getDOMWriter().write(doc, resp);
            if (logger.isInfoEnabled()) {
                elapsed2 = System.currentTimeMillis()-smillis;
                emillis = System.currentTimeMillis();
            }

            if (logger.isInfoEnabled()) logger.info("Dispatching complete! (rendered in "+(elapsed1)+"/written in "+(elapsed2)+" of "+(emillis-bmillis)+" millis)");

        } catch (ClientSideRedirectException re) {
            //if we get a redirect exception, request the browser to redirect accordingly
//            if (logger.isInfoEnabled()) logger.info("ClientSideRedirectException...redirecting to "+re.getRedirectURL());
//            resp.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
//            resp.setHeader("Location", re.getRedirectURL());

            String url = URLRewriter.encodeRedirectURL(req, resp, re.getRedirectURL());
            url = ScriptDetector.prepareRedirectURL(url, vc.getViewCapabilities());     //csc_102501.2
            if (logger.isInfoEnabled()) logger.info("ClientSideRedirectException...redirecting to "+url);
            resp.sendRedirect(url);    

        } catch (RenderException e) {
            //if we get a RenderException, handle it
            if (logger.isInfoEnabled()) logger.info("Handling RenderException:"+e);
            handleRenderException(e, vc, req, resp);
            
        } catch (EventException e) {
            //if we get an EventException, handle it
            if (logger.isInfoEnabled()) logger.info("Handling EventException:"+e);
            handleEventException(e, vc, req, resp);
            
        } finally {
            //make sure we always clean up session/local repository stuff
            ObjectRepository.removeSessionRepository();                                     //csc_022101.1
            ObjectRepository.removeLocalRepository();                                       //csc_022101.1
        }
    }
}
