/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLComponentRenderer.java,v 1.11 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.apache.log4j.*;

/**
 * This interface defines the methods needed to implement a Renderer.
 */
public class HTMLComponentRenderer extends DOMComponentRenderer {

    protected static Logger logger = Logger.getLogger(HTMLComponentRenderer.class.getName());

    protected EnabledHelper eh = new EnabledHelper();
        

    /**
     *
     */
    public Node addChildToParent(Node parent, Node child) throws InvalidNodeException {
        //eliminate the obvious
        if (parent==null || child==null) throw new InvalidNodeException("Invalid node: cannot add child:"+child+" to parent:"+parent);

        //make any adjustments specific to the markup
        if (parent instanceof HTMLElement) {
            //...<TR>
            if (parent instanceof HTMLTableRowElement) {
                if (!(child instanceof HTMLTableCellElement)) {
                    child = addChildToParent(parent.getOwnerDocument().createElement("TD"), child);
                }        
            //...<THEAD>,<TBODY>,<TFOOT>
            } else if (parent instanceof HTMLTableSectionElement) {
                if (!(child instanceof HTMLTableRowElement)) {
                    child = addChildToParent(parent.getOwnerDocument().createElement("TR"), child);
                }        
            //...<TABLE>
            } else if (parent instanceof HTMLTableElement) {
                if (!(child instanceof HTMLTableCaptionElement) &&
                    !(child instanceof HTMLTableCellElement) &&
                    !(child instanceof HTMLTableColElement) &&
                    !(child instanceof HTMLTableRowElement) &&
                    !(child instanceof HTMLTableSectionElement)) {
                    child = addChildToParent(parent.getOwnerDocument().createElement("TR"), child);
                }        
            //...<OL>,<UL>    
            } else if ((parent instanceof HTMLOListElement) ||
                       (parent instanceof HTMLUListElement)) {
                if (!(child instanceof HTMLLIElement)) {
                    child = addChildToParent(parent.getOwnerDocument().createElement("LI"), child);
                }        
            //...<OPTGROUP>
            } else if (parent instanceof HTMLOptGroupElement) {
                if (!(child instanceof HTMLOptionElement)) {
                    child = addChildToParent(parent.getOwnerDocument().createElement("OPTION"), child);
                }        
            //...<SELECT>
            } else if (parent instanceof HTMLSelectElement) {
                if (!(child instanceof HTMLOptGroupElement) &&
                    !(child instanceof HTMLOptionElement)) {
                    child = addChildToParent(parent.getOwnerDocument().createElement("OPTION"), child);
                }        
            //...<COLGROUP>    
            } else if (parent instanceof HTMLTableColElement) {
                if (!(child instanceof HTMLTableColElement)) {
                    throw new InvalidNodeException("Child:"+child+" cannot be added to parent:"+parent);
                }        
            } else {
//List pl = Classes.getAllInterfaces(parent);
//Collections.printStackTrace(pl, System.out);
//List cl = Classes.getAllInterfaces(child);
//Collections.printStackTrace(cl, System.out);
                String parentTag = ((Element) parent).getTagName().toUpperCase();
                String childTag = null;
                if (child instanceof Element) childTag = ((Element) child).getTagName().toUpperCase();
                
                //...<DL>
                if (parentTag.equals("DL")) {
                    if (!childTag.equals("DD") && !childTag.equals("DT")) {
                        child = addChildToParent(parent.getOwnerDocument().createElement("DD"), child);
                    }
                }
            }
        }
        
        //now add the child in
        if (child!=null) parent.appendChild(child);
        
        //return the parent
        return parent;
    }
    
    /**
     * The purpose of this method is to create a default Node to be used when
     * the component is not bound to any specific view. The default functionality
     * is to simply clone the template node...
     *
     * @param doc the master Document which can be used to create elements
     *        from scratch
     * @param comp the component that we're dealing with for the current request
     * @param vc the view context for the current request
     * @return a default node (created from scratch)
     * @throws UnsupportedFormatException if the renderer has no default node
     */
//csc_110501.1    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException;
    public Node createDefaultNode(Document doc, BComponent comp, ViewContext vc) throws UnsupportedFormatException {  //csc_110501.1
        //ask the renderer to create the default Node
        Node templateNode = vc.getTemplateNode();
        Node defaultNode = templateNode.cloneNode(true);        
        if (logger.isInfoEnabled()) logger.info("Creating default node:"+defaultNode);
        return defaultNode;
    }

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        Node node = view.getNode();
        String name = comp.getName();

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //do any rendering specific to all HTML components...
        //
        //name: If the component has a name, try and set it in the underlying view
        //---------------------
        if (name!=null && node!=null) {
        
            //..HTMLAnchorElement - set the "name" attribute
            if (node instanceof HTMLAnchorElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLAnchorElement interface...");
                ((HTMLAnchorElement) node).setName(name);
                
            //..HTMLAppletElement - set the "name" attribute
            } else if (node instanceof HTMLAppletElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLAppletElement interface...");
                ((HTMLAppletElement) node).setName(name);
                
            //..HTMLButtonElement - set the "name" attribute
            } else if (node instanceof HTMLButtonElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLButtonElement interface...");
                ((HTMLButtonElement) node).setName(name);
                
            //..HTMLFormElement - set the "name" attribute
            } else if (node instanceof HTMLFormElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLFormElement interface...");
                ((HTMLFormElement) node).setName(name);
                
            //..HTMLFrameElement - set the "name" attribute
            } else if (node instanceof HTMLFrameElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLFrameElement interface...");
                ((HTMLFrameElement) node).setName(name);
                
            //..HTMLIFrameElement - set the "name" attribute
            } else if (node instanceof HTMLIFrameElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLIFrameElement interface...");
                ((HTMLIFrameElement) node).setName(name);
                
            //..HTMLInputElement - set the "name" attribute
            } else if (node instanceof HTMLInputElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLInputElement interface...");
                ((HTMLInputElement) node).setName(name);
                
            //..HTMLMapElement - set the "name" attribute
            } else if (node instanceof HTMLMapElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLMapElement interface...");
                ((HTMLMapElement) node).setName(name);
                
            //..HTMLMetaElement - set the "name" attribute
            } else if (node instanceof HTMLMetaElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLMetaElement interface...");
                ((HTMLMetaElement) node).setName(name);
                
            //..HTMLObjectElement - set the "name" attribute
            } else if (node instanceof HTMLObjectElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLObjectElement interface...");
                ((HTMLObjectElement) node).setName(name);
                
            //..HTMLParamElement - set the "name" attribute
            } else if (node instanceof HTMLParamElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLParamElement interface...");
                ((HTMLParamElement) node).setName(name);
                
            //..HTMLSelectElement - set the "name" attribute
            } else if (node instanceof HTMLSelectElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLSelectElement interface...");
                ((HTMLSelectElement) node).setName(name);
                
            //..HTMLTextAreaElement - set the "name" attribute
            } else if (node instanceof HTMLTextAreaElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering "+name+" based on HTMLTextAreaElement interface...");
                ((HTMLTextAreaElement) node).setName(name);
                
            }
        }
        
        //finally, make sure we reflect the components enabled/disabled status
        //(previously this logic was in the individual renderer classes, but it
        //can really be consolidated here. Only implication: classes like 
        //HTMLActionRenderer and HTMLLink renderer must take care not to set the 
        //target/href values if the component is disabled)
//csc_021102.2 - this line was commented out, and for the life of me I don't know
//why. Consequently, I'm re-enabling it and we'll see what happens. The reason we need
//this is so that we can return a plain-jane BComponent to control visibility
        eh.setEnabled(node, comp.isEnabled());
    }
}