/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLLinkRenderer.java,v 1.15 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.apache.log4j.*;

/**
 * This class handles the default rendering of a link into an HTML view.
 */
public class HTMLLinkRenderer extends HTMLActionRenderer {

    protected static Logger logger = Logger.getLogger(HTMLLinkRenderer.class.getName());

    /**
     * The purpose of this method is to create a default Node to be used when
     * the component is not bound to any specific view. In the case of BLink,
     * it will attempt to use the default template node if possible (ie. if
     * it is an <a>, <button>, or <input> element); if anything else, it will
     * create an <a> node from scratch and append an empty text node to it
     *
     * @param doc the master Document which can be used to create elements
     *        from scratch
     * @param comp the component that we're dealing with for the current request
     * @param vc the view context for the current request
     * @return a default node (created from scratch)
     * @throws UnsupportedFormatException if the renderer has no default node
     */
//csc_110501.1    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException;
    public Node createDefaultNode(Document doc, BComponent comp, ViewContext vc) throws UnsupportedFormatException {  //csc_110501.1
        //ask the renderer to create the default Node
        Node templateNode = vc.getTemplateNode();
        Node defaultNode = null;
        if (templateNode instanceof HTMLAnchorElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLButtonElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLInputElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else {
//csc_103101.1_start - what really needs to happen here is that
//we need to clone the node (shallow), and then add the element
//node as a child to it

/*
            Element linkEl = (HTMLAnchorElement) doc.createElement("A");
            linkEl.appendChild(doc.createTextNode("foo88"));
            defaultNode = linkEl;
*/ 
/*
            Element spanEl = doc.createElement("SPAN");        
            Element linkEl = (HTMLAnchorElement) doc.createElement("A");
            Text text = doc.createTextNode("foo88");
            spanEl.appendChild(linkEl);
            linkEl.appendChild(text);
            defaultNode = spanEl;
            comp.addTempView(new DefaultView(linkEl));
*/
            //create the markup
            defaultNode = templateNode.cloneNode(false);        
            
            //csc_051502.1 - after cloning the node, we need to removed any directives embedded in its 
            //class attribute, since they won't be processed anyway
            if (defaultNode.hasAttributes()) {
                NamedNodeMap nnm = defaultNode.getAttributes();
//                for (int i=nnm.getLength()-1; i>=0; i--) {
//                    Node n = nnm.item(i);
                    Attr attr = (Attr) nnm.getNamedItem("class");
                    if (attr!=null) {
//System.out.println("here!");                    
                        String value = attr.getValue();
                        if (value!=null) {
                            StringBuffer sb = new StringBuffer(100);
                            String sep = "";
                            StringTokenizer st = new StringTokenizer(attr.getValue());
                            while (st.hasMoreTokens()) {
                                String s = st.nextToken();
                                if (!s.startsWith("Dir::")) {
                                    sb.append(sep+s);
                                    sep = " ";
                                }
                            }
                            String newValue = sb.toString().trim();
                            if (newValue.length()>0) attr.setValue(newValue);
                            else nnm.removeNamedItem("class");
                        }
//                        System.out.println("attr, name:"+attr.getName()+" val:"+attr.getValue());
                    }
//                }            
            }
                        
            Element linkEl = (HTMLAnchorElement) doc.createElement("A");
            linkEl.appendChild(doc.createTextNode("foo99"));
            try {
                addChildToParent(defaultNode, linkEl);
            } catch (InvalidNodeException e) {
                throw new UnsupportedFormatException("Error creating default link node:"+e);
            }
            
            //now create a view for the component
            comp.addTempView(new DefaultView(linkEl));
//csc_103101.1_end
        }
        if (logger.isInfoEnabled()) logger.info("Creating default node:"+defaultNode);
        return defaultNode;
    }

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a text component
        if (!(comp instanceof BLink)) throw new NoSuitableRendererException("This renderer can only render BLink components; comp is of type:"+comp.getClass().getName());
        BLink linkComp = (BLink) comp;
        Node node = view.getNode();

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //..HTMLAnchorElement
        if (node instanceof HTMLAnchorElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLAnchorElement interface...");

            //set the "target" attribute, if it exists
            String target = linkComp.getTarget();
            if (target!=null) ((HTMLAnchorElement) node).setTarget(target);

            //set the text value (if it exists) by delegating to a text component
            String text = linkComp.getText();
            if (text!=null) {
//                BText textComp = new BText(text, new DefaultView(node));
                BText textComp = new BText(text);
                textComp.setView(new DefaultView(node));
                textComp.setAllowMarkupInText(linkComp.allowMarkupInText());    //csc_092701.1
                linkComp.addStepChild(textComp, true);
            }
            
        //..HTMLButtonElement
        } else if (node instanceof HTMLButtonElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLButtonElement interface...");

            //set the text value (if it exists) by delegating to a text component
            String text = linkComp.getText();
            if (text!=null) {
//                BText textComp = new BText(text, new DefaultView(node));
                BText textComp = new BText(text);
                textComp.setView(new DefaultView(node));
                textComp.setAllowMarkupInText(linkComp.allowMarkupInText());    //csc_092701.1
                linkComp.addStepChild(textComp, true);
            }

        //..HTMLInputElement
        } else if (node instanceof HTMLInputElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLInputElement interface...");

            //set the text value (if it exists) by delegating to a text component
            String text = linkComp.getText();
            if (text!=null) {
//                BText textComp = new BText(text, new DefaultView(node));
                BText textComp = new BText(text);
                textComp.setView(new DefaultView(node));
                textComp.setAllowMarkupInText(false);   ////csc_092701.1 - input elements can't contain markup, so don't bother allowing it
                linkComp.addStepChild(textComp, true);
            }

        } else {
            String errmsg = "Node does not implement HTMLAnchorElement, HTMLButtonElement, or HTMLInputElement and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
    }
    
}