/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLListRenderer.java,v 1.15 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;

/**
 * This class handles the default rendering of a list in an HTML view.
 */
public class HTMLListRenderer extends HTMLComponentRenderer {

    protected static Logger logger = Logger.getLogger(HTMLListRenderer.class.getName());
    
    protected BList blist = null;
    protected ListModel model = null;
    protected EnabledHelper eh = new EnabledHelper();
    
    /**
     * The purpose of this method is for a renderer to provide 
     * a default node (if none exists). This component currently
     * does not provide a default, so it throws an UnsupportedFormatException 
     * instead.
     *
     * @param doc the master Document which can be used to create elements
     *        from scratch
     * @param comp the component that we're dealing with for the current request
     * @param vc the view context for the current request
     * @return a default node (created from scratch)
     * @throws UnsupportedFormatException if the renderer has no default node
     */
//csc_110501.1    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException;
    public Node createDefaultNode(Document doc, BComponent comp, ViewContext vc) throws UnsupportedFormatException {  //csc_110501.1
//        throw new UnsupportedFormatException("Cannot create default node");
        //ask the renderer to create the default Node
        Node templateNode = vc.getTemplateNode();
        Node defaultNode = templateNode.cloneNode(true);        
        if (logger.isInfoEnabled()) logger.info("Creating default node:"+defaultNode);
        return defaultNode;
    }

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a list component
        if (!(comp instanceof BList)) throw new NoSuitableRendererException("This renderer can only render BList components; comp is of type:"+comp.getClass().getName());
        blist = (BList) comp;
        model = blist.getModel();
        Node node = view.getNode();
        Node origTemplateNode = vc.getTemplateNode();   //csc_100901.4

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //HTMLElement Interface
        //---------------------
        //Supported Elements:

        //defs: fontstyle = (TT|I|B|BIG|SMALL)
        //defs: phrase = (EM|STRONG|DFN|CODE|SAMP|KBD|VAR|CITE|ABBR|ACRONYM)
        //defs: special = (A|IMG|OBJECT|BR|SCRIPT|MAP|Q|SUB|SUP|SPAN|BDO)
        //defs: formctrl = (INPUT|SELECT|TEXTAREA|LABEL|BUTTON)
        //defs: block = (P|heading|list|preformatted|DL|DIV|NOSCRIPT|BLOCKQUOTE|
        //               FORM|HR|TABLE|FIELDSET|ADDRESS)
        //defs: inline = (pcdata|fontstyle|phrase|special|formctrl)
        //defs: flow = (block|inline)

        //list structures
        //..HTMLDListElement <DL>                 (DT|DD)
        //..HTMLFieldSetElement <FIELDSET>         (pcdata|LEGEND|flow)
        //..HTMLFrameSetElement <FRAMESET>         ((FRAMESET|FRAME)+ & NOFRAMES?) 
        //..HTMLMapElement <MAP>                 (block|AREA)
        //..HTMLOList <OL>                         (LI)
        //..HTMLUList <UL>                         (LI)
        //..HTMLOptGroupElement <OPTGROUP>         (OPTION)
        //..HTMLSelectElement <SELECT>             (OPTGROUP|OPTION)
        //..HTMLTableColElement <COLGROUP>         (COL). 
        //..HTMLTableElement <TABLE>             (CAPTION?, (COL*|COLGROUP*), THEAD?, 
        //                                       TFOOT?, TBODY+)
        //..HTMLTableSectionElement <THEAD>        (TR) 
        //                            <TFOOT>        (TR) 
        //                            <TBODY>        (TR)
        //..HTMLTableRowElement <TR>            (TH|TD)

        //text containers        
        //..HTMLHeadingElement <H1>..<H6>         (inline)
        //..HTMLLabelElement <LABEL>             (inline)
        //..HTMLLegendElement <LEGEND>             (inline)
        //..HTMLLIElement <LI>                     (flow)
        //..HTMLModElement <INS>,<DEL>             (flow)
        //..HTMLObjectElement <OBJECT>             (PARAM|flow)
        //..HTMLOptionElement <OPTION>             (pcdata)
        //..HTMLParagraphElement <P>             (inline)
        //..HTMLPreElement <PRE>                 (inline, excl.IMG|OBJECT|BIG|SMALL|SUB|
        //                                                 SUP)
        //..HTMLQuoteElement <Q>                 (inline)
        //..HTMLTableCaptionElement <CAPTION>     (inline)
        //..HTMLTableCellElement <TH>,<TD>         (flow)
        //..HTMLTextAreaElement <TEXTAREA>         (pcdata)
        
        //
        //Unsupported Elements:
        //..HTMLAnchorElement, HTMLAppletElement (deprec in HTML 4.0), HTMLAreaElement,
        //....HTMLBaseElement, HTMLBaseFontElement, HTMLBodyElement, HTMLBRElement, 
        //....HTMLButtonElement, HTMLDirectoryElement (deprec in HTML 4.0),
        //....HTMLDivElement, HTMLFontElement, HTMLFormElement, HTMLFrameElement, 
        //....HTMLHeadElement, HTMLIFrameElement, HTMLImageElement, 
        //....HTMLIsIndexElement (deprec in HTML 4.0), HTMLInputElement, HTMLHRElement, 
        //....HTMLHtmlElement, HTMLLinkElement, HTMLMenuElement (deprec in HTML 4.0), 
        //....HTMLMetaElement, HTMLParamElement, HTMLScriptElement, HTMLTitleElement, 
        //....HTMLStyleElement,  
        if (node instanceof HTMLElement) {

            try {
                if (logger.isInfoEnabled()) logger.info("Rendering list component...");

                //start by removing all children from the node
                while (node.hasChildNodes()) {
                    node.removeChild(node.getFirstChild());
                }

                //give the model a chance to initialize
                model.setViewContext(vc);
                model.resetModel();
                vc.putState(ViewContext.TEMPLATE_NODE, node);

                //now get all the items from the model and add them as children
                ElementFactory ef = view.getElementFactory();
                for (int i=0,max=model.getSize(); i<max; i++) {                                
                    //get the next item
//                    Object item = model.getItemAt(i, vc);
                    Object item = model.getItemAt(i);

                    //if the item is actually a List
                    if (item instanceof List) {
                        Iterator it = ((List) item).iterator();
                        while (it.hasNext()) {
                            addItemToList(node, vc, ef, it.next(), 1);
                        }
                    //if the item is actually an array of objects
                    } else if (item instanceof Object[]) {
                        Object itemArr[] = (Object[]) item;
                        for (int j=0,jmax=itemArr.length; j<jmax; j++) {
                            addItemToList(node, vc, ef, itemArr[j], 1);
                        }                    
                    //if its just a plain old item
                    } else {
                        addItemToList(node, vc, ef, item, 1);
                    }
                }
/*                
                populateListFromModel(node, view, vc);                
*/
                
            } catch (DOMException e) {
                if (logger.isInfoEnabled()) logger.info("Node does not support children...rendering list as text", e);
            
                //convert the model to a string 
                String s = model.toString();
                
                //now try binding the string to the node via a BText component
//                BText textComp = new BText(s, new DefaultView(node));
                BText textComp = new BText(s);
                textComp.setView(new DefaultView(node));

                //now add the comp as a temporary child (by doing this, we ensure that
                //the child will get rendered as well. After rendering, we'll remove
                //the child from the hierarchy, so as to return to our original state)
//                if (tempChildren==null) tempChildren = new ArrayList();
//                tempChildren.add(textComp);
//                this.add(textComp);
                blist.addStepChild(textComp);
            } finally {
//csc_100901.4                vc.putState(ViewContext.TEMPLATE_NODE, null);
                vc.putState(ViewContext.TEMPLATE_NODE, origTemplateNode);   //csc_100901.4
                model.setViewContext(null);
            }
            
                
        } else {
            String errmsg = "Node does not implement HTMLElement and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
        
        //finally, make sure we reflect the components enabled/disabled status
        eh.setEnabled(node, comp.isEnabled());
    }
    
/*
    protected void populateListFromModel(Node node, View view, ViewContext vc) throws RenderException {
        //now get all the items from the model and add them as children
        ElementFactory ef = view.getElementFactory();
        for (int i=0,max=model.getSize(); i<max; i++) {                                
            //get the next item
            Object item = model.getItemAt(i, vc);

            //handle it
            handleNextItem(node, ef, item);
        }
    }

    protected void handleNextItem(Node node, ElementFactory ef, Object item) throws RenderException {
        //if the item is actually a List
        if (item instanceof List) {
            Iterator it = ((List) item).iterator();
            while (it.hasNext()) {
                addItemToList(node, ef, it.next(), 1);
            }
        //if the item is actually an array of objects
        } else if (item instanceof Object[]) {
            Object itemArr[] = (Object[]) item;
            for (int j=0,jmax=itemArr.length; j<jmax; j++) {
                addItemToList(node, ef, itemArr[j], 1);
            }                    
        //if its just a plain old item
        } else {
            addItemToList(node, ef, item, 1);
        }
    }
*/    
    protected void addItemToList(Node node, ViewContext vc, ElementFactory ef, Object item, int depth) throws RenderException {

        //show what the item implements                    
        if (logger.isDebugEnabled()) {
            //csc_102601.1_start - this causes a null pointer exception if 
            //the item happens to be a BComponent which isnot yet bound to a view
//            if (logger.isDebugEnabled()) logger.debug("item "+item+" implements the following interfaces:");
            if (logger.isDebugEnabled()) {
                if (item instanceof BComponent) logger.debug("item " + item.getClass().getName() + "@" + Integer.toHexString(item.hashCode()) + " implements the following interfaces:");
                else logger.debug("item "+item+" implements the following interfaces:");
            }
            //csc_102601.1_end
            Iterator it = Classes.getAllInterfaces(item).iterator();
            while (it.hasNext()) {
                Object o = it.next();
                if (logger.isDebugEnabled()) logger.debug(o.toString());
            }
        }

        //get it back into the DOM
        //..BComponent
        if (item instanceof BComponent) {
            //add in the Node for each view
            if (logger.isDebugEnabled()) logger.debug("Getting next BComponent item: "+item+"...");
            BComponent wcomp = (BComponent) item;

/*
//Orig            
            Iterator it = wcomp.getViews().iterator();
            while (it.hasNext()) {
                View view = (View) it.next();
                addChildToParent(node, view.getNode());
            }
*/
            //if the component needs a default view create one for it...
//csc_110501.1            List views = wcomp.getViews();
//csc_110501.1            if (views==null || views.size()<1) {
            if (!wcomp.hasViews()) {                                    //csc_110501.1
                //get the format type and the doc from the view context
                FormatType ft = vc.getViewCapabilities().getFormatType();
                Document doc = vc.getElementFactory().getDocument();

                try {
                    //get the appropriate renderer by looking up the
                    //DOM class associated with the given format type
                    Renderer r = wcomp.getRenderer(ft.getDOMClass());
                    

//csc_110501.1_start
/*
                    //ask the renderer to create the default Node
                    Node n = r.createDefaultNode(doc, vc);
                    addChildToParent(node, n);
                    
                    //finally create a view on the resulting node and add it in
                    wcomp.addTempView(new DefaultView(n));
                    wcomp.invalidate();
*/                    
                    //ask the renderer to create the default Node
                    Node n = r.createDefaultNode(doc, wcomp, vc); //csc_110501.1
                    addChildToParent(node, n);
                    
                    //if the component still needs a default view create one for it...
                    if (!wcomp.hasViews()) wcomp.addTempView(new DefaultView(n));
                    wcomp.invalidate();
//csc_110501.1_end
                    
                } catch (RenderException e) {
                    logger.warn("Unable to create default view:", e);
                } catch (DOMException e) {
                    logger.warn("Unable to create default view:", e);
                }
                
            //if the component being returned is already bound to views,
            //just add in the nodes that back the views
//csc_110501.1            } else if (views!=null) {
//csc_110501.1                Iterator it = views.iterator();
            } else {                                                    //csc_110501.1
                Iterator it = wcomp.getViews().iterator();              //csc_110501.1
                while (it.hasNext()) {
                    View view = (View) it.next();
                    addChildToParent(node, view.getNode());
                }
            }
            
            //now add the comp as a temporary child (by doing this, we ensure that
            //the child will get rendered as well. After rendering, we'll remove
            //the child from the hierarchy, so as to return to our original state)
//            if (tempChildren==null) tempChildren = new ArrayList();
//            tempChildren.add(wcomp);
//            this.add(wcomp);
            blist.addStepChild(wcomp);

        //..Nodes
        } else if (item instanceof Node) {
            if (logger.isDebugEnabled()) logger.debug("Getting next Node item: "+item+"...");
//            node.appendChild((Node) item);
            addChildToParent(node, (Node) item);
        
        //..Strings
        } else {
            if (logger.isDebugEnabled()) logger.debug("Getting next String item: "+item+"...");
//by making ItemMap implement toString(), we don't even have to explicitly reference it here...
//            String s = null;
//            if (item instanceof ItemMap) s = ((ItemMap) item).getValue().toString();
//            else s = item.toString();                    
            String s = item.toString();
            
            //get the template for this particular node
            Node templateNode = ef.getDefaultElement();
            Node newNode = null;
            if (templateNode!=null) {
                //create the new node by cloning
                newNode = templateNode.cloneNode(true);
            } else {
                //create the new node from scratch
                String targetEl = null;
                if (node instanceof HTMLDListElement) targetEl = "DT";
                else if (node instanceof HTMLOListElement) targetEl = "LI";
                else if (node instanceof HTMLOptGroupElement) targetEl = "OPTION";
                else if (node instanceof HTMLSelectElement) targetEl = "OPTION";
                else if (node instanceof HTMLTableColElement) targetEl = "COL";
                else if (node instanceof HTMLTableRowElement) targetEl = "TD";
                else if (node instanceof HTMLUListElement) targetEl = "LI";
                else throw new DOMException((short) 0, "Unsupported List node");
                newNode = node.getOwnerDocument().createElement(targetEl);
            }
            
            //now populate the new node and add it back in
            if (newNode!=null) {
//                BText textComp = new BText(s, new DefaultView(newNode));
                BText textComp = new BText(s);
                textComp.setView(new DefaultView(newNode));
//                if (tempChildren==null) tempChildren = new ArrayList();
//                tempChildren.add(textComp);
//                this.add(textComp);
                blist.addStepChild(textComp);
//                node.appendChild(newNode);
                addChildToParent(node, newNode);
                
            } else {
                throw new InvalidNodeException("Unable to create a new Text node");
            }
        }
    }
    
}