/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EventGateway.java,v 1.5 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.util.*;

/**
 * An EventGateway simply represents a gateway to a set of event 
 * handlers within a system. 
 *
 * Gateways are heirarchical in that they may contain other 
 * EventGateways, and may have a parent gateway. Invoking register 
 * on a gateway should cause it to 
 *
 * a) register all known enties that are interested in receiving 
 *    events from the EventBroker
 * b) invoke register for any gateways it contains
 *
 * Invoking deregister has the opposite effect.
 */
public interface EventGateway {

    /**
     * Set the parent event gateway. Null indicates its the root.
     *
     * @param eg the parent event gateway
     */
    public void setParent(EventGateway eg);

    /**
     * Get the parent event gateway. Returns null if it's the root.
     *
     * @return the parent event gateway
     */
    public EventGateway getParent();

    /**
     * Add an event gateway to this one
     *
     * @param eg the event gateway to be added
     */
    public void add(EventGateway eg);

    /**
     * Remove an event gateway from this one
     *
     * @param eg the event gateway to be removed
     */
    public void remove(EventGateway eg);

    /**
     * Get a list of child gateways
     *
     * @return a list of child gateways
     */
    public List getChildren();

    /**
     * Ask all interested parties to register with
     * the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        register for events
     */
    public void register(EventBroker eb);

    /**
     * Ask all interested parties to de-register with
     * the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        de-register for events
     */
    public void deregister(EventBroker eb);

}
