/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultViewHandler.java,v 1.11 2002/06/13 14:55:01 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event.helper;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.util.http.*;
import org.apache.log4j.*;

/**
 * <p>A very simple view handler that provides a default implementation
 * tailored for the use of components. In many ways, this is analagous to
 * the ComponentGateway class--a component hierarchy is created, initialized
 * and rendered automatically for you. All the developer has to do is a)
 * implement the handleViewEvent method to add any custom components to the root
 * component (ie. for rendering) and b) return the master DOM page that needs
 * to be rendered to generate the final view.
 */
public abstract class DefaultViewHandler extends DefaultBaseEventListener {

    //public vars...eventually, these should probably be final
    protected static Logger localLogger = Logger.getLogger(DefaultViewHandler.class.getName());
    
    //private vars
    //csc_061202.1_start - add support for max age; also note that these should NOT be global
    //as they will cause the pretty printing and caching stuff to apply to the entire app, 
    //and realistically, we probably wouldn't want to do that...we'd probably want to do
    //it on a page level instead
    protected boolean printPretty = false;
    protected boolean preventCaching = false;
    protected int maxAge = 0;
    //csc_061202.1_end
    protected boolean recycleChildren = false;

    /**
     * Public noargs constructor
     */
    public DefaultViewHandler() {}


    //-------------------- DefaultViewHandler --------------------
    /**
     * <p>Handle a view event. This is the method developers will 
     * typically override. The developers shaould add any components
     * to the root component and then return the underlying DOM Document 
     * (that backs their components) so it can be rendered
     *
     * @param root the root component which will get rendered as a result 
     *         of this request
     * @param vc the ViewContext object describes what features the 
     *         client view is capable of supporting
     * @return the Document to be rendered
     * @throws ServletException
     * @throws IOException
     */
    public abstract Document handleViewEvent(BComponent root, ViewContext vc) throws EventException, ServletException, IOException;
    
    /**
     * The purpose of this method is to allow for optional pre-component-render 
     * cycle processing (ie. to stick a value in the user's session)
     * @param root the root component which will get rendered as a result 
     *         of this request
     * @param vc the ViewContext object describes what features the 
     *         client view is capable of supporting
     */
    public void preCompRender(BComponent root, ViewContext vc) {
        //(--n/a--)
    }
    
    /**
     * The purpose of this method is to allow for optional post-component-render 
     * cycle processing (ie. to remove a value from the user's session)
     * @param root the root component which will get rendered as a result 
     *         of this request
     * @param vc the ViewContext object describes what features the 
     *         client view is capable of supporting
     */
    public void postCompRender(BComponent root, ViewContext vc) {
        //(--n/a--)
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        DefaultDOMWriter ddw = new DefaultDOMWriter(printPretty, preventCaching);
        ddw.setMaxAge(maxAge);  //csc_061202.1
        return ddw;
    }


    //-------------------- DefaultBaseEventListener --------------
    /**
     * Handle the ViewEvent
     */
    public void handleViewEvent(ViewEventContext vec) throws EventException, ServletException, IOException {
        long bmillis = 0;
        long smillis = 0;
        long emillis = 0;
        long elapsed1 = 0;
        long elapsed2 = 0;
        if (localLogger.isInfoEnabled()) bmillis = System.currentTimeMillis();
        if (localLogger.isInfoEnabled()) localLogger.info("Handling ViewEvent in "+this);

        try {
            //start by figuring out the ViewCapabilities
            if (localLogger.isDebugEnabled()) localLogger.debug("Create the ViewContext");
            ViewContext vc = new DefaultViewContext(vec);
    
            //create our root component
            if (localLogger.isDebugEnabled()) localLogger.debug("Create component root");
            BComponent broot = new BComponent();
            broot.setName("Root");
        
            //give the implementation a chance to add any components to the root
            if (localLogger.isDebugEnabled()) localLogger.debug("Handling default");
    //            Document doc = handleDefault(broot, vc, req, resp);
            Document doc = handleViewEvent(broot, vc);
        
            //now init the component
            if (localLogger.isDebugEnabled()) localLogger.debug("Invoking initCycle on component hierarchy");
            broot.initCycle();
        
            //allow for pre-rendering processing //csc_021102.2 - added
            if (localLogger.isDebugEnabled()) localLogger.debug("Pre-component render");
            if (localLogger.isDebugEnabled()) smillis = System.currentTimeMillis();
            preCompRender(broot, vc);
            if (localLogger.isDebugEnabled()) elapsed1 = System.currentTimeMillis()-smillis;

            //now render the component
            if (localLogger.isDebugEnabled()) localLogger.debug("Rendering component hierarchy");
            if (localLogger.isDebugEnabled()) smillis = System.currentTimeMillis();
            broot.render(vc);
            if (localLogger.isDebugEnabled()) elapsed1 = System.currentTimeMillis()-smillis;

            //allow for pre-rendering processing //csc_021102.2 - added
            if (localLogger.isDebugEnabled()) localLogger.debug("Post-component render");
            if (localLogger.isDebugEnabled()) smillis = System.currentTimeMillis();
            postCompRender(broot, vc);
            if (localLogger.isDebugEnabled()) elapsed1 = System.currentTimeMillis()-smillis;

            //its possible the implementor may want to recycle children...if so, remove them
            //prior to calling destroy on the root
            if (recycleChildren) {
                if (localLogger.isDebugEnabled()) localLogger.debug("Recycling child components");
                List children = broot.getChildren();
                if (children!=null) {
                    for (int i=children.size()-1; i>=0; i--) {
                        broot.removeChild(i);            
                    }
                }
            }

            //now destroy the component
            if (localLogger.isDebugEnabled()) localLogger.debug("Invoking destroyCycle on component hierarchy");
            broot.destroyCycle();
        
            //csc_102201.1            
            //now adjust the outgoing page (this is critical to make sure we
            //can accurately detect client scripting)
            ScriptDetector.prepareClientResp(doc, vc);

            //now render the page
            if (localLogger.isDebugEnabled()) localLogger.debug("Rendering the DOM");
            if (localLogger.isInfoEnabled()) smillis = System.currentTimeMillis();
            this.getDOMWriter().write(doc, vec.getResponse());
            if (localLogger.isInfoEnabled()) {
                elapsed2 = System.currentTimeMillis()-smillis;
                emillis = System.currentTimeMillis();
            }
        } catch (RenderException e) {
            //if we get an EventException, handle it
            if (localLogger.isInfoEnabled()) localLogger.info("Unexpected RenderException:"+e);
            throw new EventException ("Unexpected RenderException:"+e, e);
            
        } finally {
            //nop
        }

        if (localLogger.isInfoEnabled()) localLogger.info("ViewEvent handled! (rendered in "+(elapsed1)+"/written in "+(elapsed2)+" of "+(emillis-bmillis)+" millis)");
    }


}
