/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s): Christian Cryder, Diez B. Roggisch, Jacob Kjome
 * 
 * $Id: FormUtil.java,v 1.1 2002/03/15 16:44:35 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;

/**
 * <p>Simple Form related utilities
 * 
 * @author  Christian Cryder <christianc@enhydra.org>
 * @version %I%, %G%
 * @since   1.0
 */
public class FormUtil {

    /**
     * <p>This method allows you to assert that a String value falls within a given 
     * min/max length range. If min or max is -1, that particular aspect will not 
     * be evaluated.
     *
     * <p>Example usage: <br>
     *    String user = assertMinMax((String) map.get(USER), "User Name", 5, 30);
     *
     * <p>This would retrieve a username from a map, validate it to ensure that its
     * between 5 and 30 characters in length, and then return the adjusted value (if it
     * was null in the map, it will come back as "", which is convenient for further 
     * custom evaluation)
     *
     * <p>Todo:
     * <ul>
     *   <li>The error message generated by this method is not localized yet
     *   </li>
     * </ul>
     *
     * @param field the field to be evaluated
     * @param fieldDescr a description of the field to be included in any error messages
     * @param min the minimum length of the String, or -1 if there is no min
     * @param max the maximum length of the String, or -1 if there is no max
     * @return the field value (non-null, adjusted and trimmed for further custom evaluation)
     * @throws a ValidationException if the field is not valid
     */
    public static String assertMinMax(String field, String fieldDescr, int min, int max) throws ValidationException {
        if (field==null) field = "";
        else field = field.trim();
        if (min!=-1) assertTrue(fieldDescr+" must be at least "+min+" chars in length -- Please re-enter!", field.length()>=min);
        if (max!=-1) assertTrue(fieldDescr+" may not exceed "+max+" chars in length -- Please re-enter!", field.length()<=max);
        return field;
    }
    
    /**
     * This function simply evaluates a given boolean expression and throws a ValidationException
     * using the specified error message if its not valid
     *
     * @param errmsg the error message to be used if the expression is invalid
     * @param expression the expression to be evaluated
     * @throws ValidationException if the expression is not true
     */
    public static void assertTrue(String errmsg, boolean expression) throws ValidationException {
        if (!expression) throw new ValidationException(null, errmsg);
    }


}
