/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ObjectRepositoryAssembler.java,v 1.7 2002/07/21 21:41:03 JacobK Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.io.*;
import java.lang.*;
import java.lang.reflect.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.util.xml.*;


/**
 * This class basically assembles objects into the default ObjectRepository based on 
 * an XML descriptor file. For a sample file look at /WEB-INF/object-repository.xml
 *
 * <p>There are only a few basic tags:
 * <ul>
 *   <li>
 *      <strong>object</strong> - this tag asks the assembler to instantiate an
 *          object, using the name, class, and arg parameters specified. The object
 *          will be instantiated directly after the start tag is processed (ie. before
 *          any method tags, since the object must exist in order for the methods
 *          to be invoked).
 *
 *      <p>Attributes:<ul>
 *        <li>
 *           <strong>name</strong> - If you specify an object name (ie. '$obj1'), 
 *              a reference to the object will be saved for the duration of the 
 *              assembly process, allowing you to refer to the object elsewhere 
 *              (ie. as a parameter of another method).
 *        </li><li>
 *           <strong>class</strong> - The name of the object to be instantiated
 *        </li><li>
 *           <strong>arg, arg0...arg9</strong> - You can pass args into the object
 *              constructor as well if need. Frequently, the args might refer to other
 *              named objects.
 *        </li>
 *      </ul>
 *   </li>
 *   <li>
 *      <strong>method</strong> - this tag invokes a method on the object currently
 *          in scope. The method is not actually invoked until the end tag is hit 
 *          (allowing us to get any param tags that might accompany the method).
 *
 *      <p>Attributes:<ul>
 *        <li>
 *           <strong>name</strong> - name of the method to be invoked
 *        </li><li>
 *           <strong>arg, arg0...arg9</strong> - You can pass args into the object
 *              constructor as well if need. Frequently, the args might refer to other
 *              named objects.
 *        </li>
 *      </ul>
 *
 *      <p>In addition to using arg attributes, you can use the value of the method
 *          tag (ie. if you only have one param), or you can use multiple param
 *          tags. 
 *   </li>
 *   <li>
 *      <strong>prop</strong> - this tag allows you to set a property in an object (static
 *          or otherwise). The property is not actually set until the end tag is hit 
 *          (allowing us to get any param tags that might accompany the prop).
 *
 *      <p>Attributes:<ul>
 *        <li>
 *           <strong>name</strong> - name of the property to be set
 *        </li>
 *      </ul>
 *
 *      <p>Unlike the method attribute, prop doesn't take arguments. Instead, you use the
 *          value of the prop to specify what the value should be set to. Right now, Strings,
 *          Integers, and Booleans are supported.
 *   </li>
 *   <li>
 *      <strong>param</strong> - this tag specifies a parameter for a method. Note that
 *          the param type checking is not very sophisticated; if the param is an instance
 *          of a String (as opposed to a reference to a named object), the assembler 
 *          will try it first as an int (if possible), else as a String.
 *
 *      <p>Attributes: n/a
 *   </li>
 *   <li>
 *      <strong>register</strong> - this tag asks the assembler to place the specified
 *          object into the default object repository.
 *
 *      <p>Attributes:<ul>
 *        <li>
 *           <strong>key</strong> - the key value which may be used to retrieve the object
 *              from the default object repository
 *        </li><li>
 *           <strong>val</strong> - a reference to the item to be placed in the default
 *              object repository (ie. $obj1)
 *        </li>
 *      </ul>
 *   </li>
 * </ul>
 
 * <p>Shortcomings and areas for improvement:
 * <ul>
 *   <li>
 *      It would be nice if when resolving name refences you could not only 
 *      refer to names, but also to methods: ie. $foo.bar() --> foo.bar(), 
 *      foo.arg[1].get("blah") --> find the arg property in foo (it should be an
 *      array), take element 1, and try to invoke a .get("blah") method on it.
 *   </li><li>
 *      The param/arg type conversion isn't very intelligent right now (ie. it'll
 *      handle Strings, ints, and Object types, but nothing else
 *   </li><li>
 *      Not sure if it handles static methods yet
 *   </li>
 * </ul>
 */
public class ObjectRepositoryAssembler extends HttpServlet {
    
    //public constants
    protected static Logger logger = Logger.getLogger(ObjectRepositoryAssembler.class.getName());
    
    //private constants
    public static String ASSEMBLY_DESCRIPTOR = "AssemblyDescriptor";
    public static String SAX_PARSER = "SAXParser";
    public static String DEFAULT_DESCRIPTOR = "object-repository.xml";
    public static String DEFAULT_PARSER = "org.apache.xerces.parsers.SAXParser";

    //recognized tags
    private static final String OBJECT = "object";
    private static final String METHOD = "method";
    private static final String PROP = "prop";
    private static final String PARAM = "param";
    private static final String REGISTER = "register";
    
    //recognized attributes
    private static final String NAME = "name";
    private static final String CLASS = "class";
    private static final String ARG = "arg";
    private static final String KEY = "key";
    private static final String VAL = "val";
    //private vars
    protected String parserClass = null;
    protected String assemblySourceFile = null;
    protected ObjectRepository or = null;
    protected ServletConfig servletConfig = null;
    protected Map objMap = new HashMap();
    protected List argList = new ArrayList();
    protected Stack objStack = null;
    protected String methodName = null;
    protected String propName = null;   //csc_052902.1 - added
    protected boolean needPropVal = false;
    protected String propVal = null;
    protected int paramCntr = -1;
    protected LocalAssemblyHandler handler = null;
/*    
    protected EventGateway rootGateway = null;
    protected EventGateway curGateway = null;
    protected String className = null;
    protected String propName = null;
*/
    protected int depth = 0;

    public static String TEST_STRING = null;
    public static int TEST_INT = -1;
    public static boolean TEST_BOOLEAN = false;
        
    //--------------- ObjectRepositoryAssembler ------------------
    /**
     * Assemble the system into the default ObjectRepository given the
     * XML assembly decriptor name. The default parser will be 
     * used.
     *
     * @param assemblySourceFile the XML assembly descriptor (if null defaults
     *      to DEFAULT_DESCRIPTOR)
     */
    public void assemble(String iassemblySourceFile) {
        assemble(null, iassemblySourceFile);
    }
    
    /**
     * Assemble the system into the default ObjectRepository given the
     * XML assembly decriptor name. The default parser will be 
     * used.
     *
     * @param servletConfig the ServletConfig object (may be null if you are
     *      calling this from other than a servlet environment)
     * @param assemblySourceFile the XML assembly descriptor (if null defaults
     *      to DEFAULT_DESCRIPTOR)
     */
    public void assemble(ServletConfig iservletConfig, String iassemblySourceFile) {
        assemble(null, iservletConfig, iassemblySourceFile);
    }
    
    /**
     * Assemble the system, given a specified ObjectRepository and the
     * XML assembly decriptor name. The default parser will be 
     * used.
     *
     * @param or the repository we wish to assemble into (if null 
     *      defaults to default ObjectRepository)
     * @param servletConfig the ServletConfig object (may be null if you are
     *      calling this from other than a servlet environment)
     * @param assemblySourceFile the XML assembly descriptor (if null defaults
     *      to DEFAULT_DESCRIPTOR)
     */
    public void assemble(ObjectRepository ior, ServletConfig iservletConfig, String iassemblySourceFile) {
        assemble(ior, iservletConfig, iassemblySourceFile, null);
    }
    
    /**
     * Assemble the system, given the root EventGateway, an
     * XML assembly decriptor name, and a specific SAX parser
     * class.
     *
     * @param or the repository we wish to assemble into (if null 
     *      defaults to default ObjectRepository)
     * @param servletConfig the ServletConfig object (may be null if you are
     *      calling this from other than a servlet environment)
     * @param assemblySourceFile the XML assembly descriptor (if null defaults
     *      to DEFAULT_DESCRIPTOR)
     * @param parserClass the SAX parser class (if null, defaults to DEFAULT_PARSER)
     */
    public void assemble(ObjectRepository ior, ServletConfig iservletConfig, String iassemblySourceFile, String iparserClass) {
        or = ior;
        if (or==null) or = ObjectRepository.getGlobalRepository();
        if (iassemblySourceFile==null) iassemblySourceFile = DEFAULT_DESCRIPTOR;
        if (iparserClass==null) iparserClass = DEFAULT_PARSER;
        servletConfig = iservletConfig;
        ServletContext context = null;
        if (servletConfig!=null) context = servletConfig.getServletContext();
        assemblySourceFile = iassemblySourceFile;
        parserClass = iparserClass;

        String path = (context!=null ? context.getRealPath("/") : "");
        String astr = path+assemblySourceFile.replace('/', File.separatorChar);
        logger.info("Configuring "+or.getName()+" from File: "+astr);
        if (context==null) System.out.println("Configuring "+or.getName()+" from File: "+astr);
        else context.log("Configuring "+or.getName()+" from File: "+astr);

        try {
            //instantiate the parser
            logger.info("Instantiating parser ("+parserClass+")");
//jrk_20020414.1_start
            //Class.forName is java1. Since Java2, use 
            //Thread.currentThread().getContextClassLoader().loadClass("MyClass")
            //This should fix some classloading issues in Engines with multiple
            //class loaders (eg.. Tomcat-3.3.x and Tomcat-4.x.x)
            //XMLReader parser = (XMLReader) Class.forName(parserClass).newInstance();
            XMLReader parser = (XMLReader) Thread.currentThread().getContextClassLoader().loadClass(parserClass).newInstance();
//jrk_20020414.1_end
//            parser.setContentHandler(this);
//            parser.setErrorHandler(this);
            handler = new LocalAssemblyHandler(or);
            parser.setContentHandler(handler);
            parser.setErrorHandler(handler);

            //if we have access to the ServletConfig info, try getting
            //the input stream with the Servlet classloader (this will allow
            //us to read from the WEB-INF directory)
            InputStream is = null;
            if (context!=null) {        
                logger.info("Getting input source from Servlet ("+assemblySourceFile+")");
                is = context.getResourceAsStream(assemblySourceFile);
            }

            //if we couldn't find the file there, look in the /classes directory            
            if (is==null) {
                logger.info("Getting input source from Assembler ("+assemblySourceFile+")");
                is = this.getClass().getResourceAsStream(assemblySourceFile);
            }
            
            //if we couldn't find the file there, try looking for it using a FileInputStream
            if (is==null) {
                logger.info("Getting input source as File ("+assemblySourceFile+")");
                File f = new File(assemblySourceFile);
                if (f.exists()) is = new FileInputStream(f);
            }
            
            //get the source file
            InputSource source = new InputSource(is);
            
            //parse the file            
            logger.info("Parsing the source file...");
            parser.parse(source);
            
        } catch (org.xml.sax.SAXParseException spe) {
            logger.warn("Error assembling system!", spe);
            spe.printStackTrace(System.err);
        } catch (org.xml.sax.SAXException se) {
            if (se.getException() != null) {
                logger.warn("Error assembling system!", se.getException());
                se.getException().printStackTrace(System.err);
            } else {
                logger.warn("Error assembling system!", se);
                se.printStackTrace(System.err);
            }
        } catch (Exception e) {
            logger.warn("Error assembling system!", e);
            e.printStackTrace(System.err);
        }

    }
    
    
    
    //--------------- DefaultHandler -----------------------------
    public class LocalAssemblyHandler extends DefaultHandler {
        ObjectRepository or = null;

        public LocalAssemblyHandler(ObjectRepository ior) {
            or = ior;
        }

        public void startDocument() {
            objStack = new Stack();
        }

        public void startElement(String uri, String local, String raw, Attributes attrs) throws SAXException {
            String curTag = local;
            logger.debug("Starting w/: " + curTag);
    //        logger.debug("uri:"+uri+" local:"+local+" raw:"+raw+" attrs:"+attrs);        

            //object - we need to create a new object and store it in the object map
            if (curTag.equals(OBJECT)) {
        
                //start by getting the attrs and any args
                depth++;
                String objname = attrs.getValue(NAME);
                String clname = attrs.getValue(CLASS);
                Object arg = resolve(attrs.getValue(ARG));
                if (arg!=null) argList.add(arg);
                for (int i=0; i<10; i++) {
                    arg = resolve(attrs.getValue(ARG+i));
                    if (arg!=null) argList.add(arg);
                }
                if (logger.isDebugEnabled()) {
                    logger.debug("Creating obj--> objname:"+objname+" clname:"+clname);
                    if (argList.size()>0) {
                        String s = "    args:";
                        String sep = "";
                        for (int i=0; i<argList.size(); i++) {
                            s = s+sep+argList.get(i);
                            sep = ", ";
                        }
                        logger.debug(s);
                    }
                }
            
                //now try and instantiate it
                Class cl = null;
                Object obj = null;
                try {
//jrk_20020414.2_start
                    //Class.forName is java1. Since Java2, use 
                    //Thread.currentThread().getContextClassLoader().loadClass("MyClass")
                    //This should fix some classloading issues in Engines with multiple
                    //class loaders (eg.. Tomcat-3.3.x and Tomcat-4.x.x)
                    //cl = Class.forName(clname);
                    cl = Thread.currentThread().getContextClassLoader().loadClass(clname);
//jrk_20020414.2_end
                    if (argList.size()<1) {
                        obj = cl.newInstance();
                    } else {
                        Object args[] = new Object[argList.size()];
                        Class argcl[] = new Class[argList.size()];
                        for (int i=0; i<argList.size(); i++) {
                            args[i] = argList.get(i);
                            argcl[i] = argList.get(i).getClass();
                        }
                    
                        if (logger.isDebugEnabled()) {
                            StringBuffer sb = new StringBuffer("Looking for constructor "+clname+"{");
                            String sep = "";
                            for (int i=0; i<argList.size(); i++) {
                                sb.append(sep+argcl[i].getName());
                                sep = ", ";
                            }
                            sb.append("}");
                            logger.debug(sb.toString());
                        }
                        Constructor constructor = null;
                        try {
                            constructor = cl.getConstructor(argcl);
                        } catch (Exception e) {
                        }
                        obj = constructor.newInstance(args);
                    }
                } catch (Exception e) {
                    String msg = "Error instantiating object";
                    logger.warn(msg+": ", e);
                    throw new SAXException(msg, e);
                }
      
                //push the object onto the stack
                objStack.push(obj);
            
                //finally, if there was a name register the object accordingly
                if (objname!=null) {
                    objMap.put(objname, obj);
                }

            //method - invoke a method
            } else if (curTag.equals(METHOD)) {
                methodName = attrs.getValue(NAME);
                Object arg = resolve(attrs.getValue(ARG));
                if (arg!=null) argList.add(arg);
                for (int i=0; i<10; i++) {
                    arg = resolve(attrs.getValue(ARG+i));
                    if (arg!=null) argList.add(arg);
                }
                propVal = null;
                needPropVal = true;
                paramCntr = -1;
            
            //csc_052902.1 - added
            //prop - set a property
            } else if (curTag.equals(PROP)) {
                propName = attrs.getValue(NAME);
                propVal = null;
                needPropVal = true;
                paramCntr = -1;

            //param - parameter (add to args list)
            } else if (curTag.equals(PARAM)) {
                //the first time we hit a param tag after a method tag,
                //reset the argList (because it may contain a blank parameter)
                if (++paramCntr==0) argList = new ArrayList();
                propVal = null;
                needPropVal = true;

            //register - register an object in the repository
            } else if (curTag.equals(REGISTER)) {
                String key = attrs.getValue(KEY);
                Object val = resolve(attrs.getValue(VAL));
                or.putState(key, val);
            }
        }

        public void endElement(String uri, String local, String raw) throws SAXException {
            String curTag = local;
        
            //object
            if (curTag.equals(OBJECT)) {
                //pop the object back off the stack
                objStack.pop();
                argList = new ArrayList();
                depth--;
        
            //param
            } else if (curTag.equals(PARAM)) {
            
            //method - here's where we actually invoke the method
            } else if (curTag.equals(METHOD)) {
                StringBuffer mthsb = new StringBuffer(methodName+"{");
                if (argList.size()>0) {
                    String s = "    args:";
                    String sep = "";
                    for (int i=0; i<argList.size(); i++) {
                        mthsb.append(sep+argList.get(i));
                        sep = ", ";
                    }
                    mthsb.append("}");
                }
                logger.debug("Invoking method--> "+mthsb.toString());
        
                //get the target object
                Object targetObj = objStack.peek();
                List clList = new ArrayList();
                List mthList = new ArrayList();
                Class ocl = targetObj.getClass();
                while (true) {
                    Method allMethods[] = ocl.getDeclaredMethods();
                    for (int i=0; i<allMethods.length; i++) {
                        Method m = allMethods[i];
                        if (!m.getName().equals(methodName)) continue;
                        int mod = m.getModifiers();
                        if (!Modifier.isPublic(mod)) continue;
                        Class paramCl[] = m.getParameterTypes();
                        if (paramCl.length!=argList.size()) continue;
                        logger.debug("found possible method:"+m);
                        mthList.add(m);
                        clList.add(ocl);
                    }
                    if (ocl==Object.class) break;
                    ocl = ocl.getSuperclass();
                }
                if (mthList.size()<1) {
                    String msg = "Unable to find matching method: "+mthsb.toString();
                    logger.warn(msg);
                    throw new SAXException(msg);
                }
        
                //now we need to iterate through the methods we found and see if we
                //can invoke them
                boolean success = false;
                for (int i=0; i<mthList.size(); i++) {
                    //start by trying to convert the params into the target types
                    Method m = (Method) mthList.get(i);
                    Class paramTypes[] = m.getParameterTypes();
                    Object args[] = new Object[paramTypes.length];
                    for (int j=0; j<argList.size(); j++) {
                        Object arg = argList.get(j);
                        if (arg instanceof String) {
                            String sarg = (String) arg;
                            if (paramTypes[j]==int.class) {
                                args[j] = new Integer(sarg);
                            } else {
                                args[j] = arg;
                            }
                        } else {
                            args[j] = arg;
                        }
                    }
                
                    //now try and invoke the method
                    try {
                        m.invoke(targetObj, args);
                        success = true;
                        logger.debug("successfully invoked: "+m);
                        break;
                    } catch (Exception e) {
                        logger.debug("error invoking method m:"+e);
                        continue;
                    }
                }
                if (!success) {
                    String msg = "Unable to invoke method: "+mthsb.toString();
                    logger.warn(msg);
                    throw new SAXException(msg);
                }
        
                //make sure we clean up the argList
                argList = new ArrayList();
            
            //csc_052902.1 - added
            //prop - here's where we actually set the property
            } else if (curTag.equals(PROP)) {
                logger.debug("Setting prop--> "+propName+"="+propVal);
        
                //get the target object
                Object targetObj = objStack.peek();
                Field fld = null;
                Class ocl = targetObj.getClass();
                while (true) {
                    Field allFields[] = ocl.getDeclaredFields();
                    for (int i=0; i<allFields.length; i++) {
                        Field f = allFields[i];
                        if (f.getName().equals(propName) && Modifier.isPublic(f.getModifiers())) {
                            logger.debug("found property:"+f);
                            fld = f;
                            break;
                        }
                    }
                    if (ocl==Object.class) break;
                    ocl = ocl.getSuperclass();
                }
                if (fld==null) {
                    String msg = "Unable to find matching property: "+propName;
                    logger.warn(msg);
                    throw new SAXException(msg);
                }

                //now we need to set the property
                try {
                    Object iprop = fld.get(targetObj);
                    Object targetClass = (iprop!=null ? iprop.getClass() : String.class);
                    Object targetProp = null;
                    if (propVal!=null) {
                        //..String
                        if (targetClass.equals(String.class)) {
                            logger.debug("Setting String value");
                            targetProp = new String(propVal);
                            fld.set(targetObj, targetProp);                    
                        //..Integer
                        } else if (targetClass.equals(Integer.class)) {
                            logger.debug("Setting Integer value");
                            Integer i = new Integer(propVal);
                            fld.setInt(targetObj, i.intValue());                    
                        //..Boolean
                        } else if (targetClass.equals(Boolean.class)) {
                            logger.debug("Setting Boolean value");
                            String tpropVal = propVal.toLowerCase().trim();
                            boolean val = (tpropVal.equals("true") || tpropVal.equals("yes") || tpropVal.equals("on") || tpropVal.equals("1"));
                            fld.setBoolean(targetObj, val);                    
                        //..everything else
                        } else {
                            logger.debug("Setting value (default)");
                            fld.set(targetObj, targetProp);
                        }
                    } else {
                        fld.set(targetObj, null);
                    }
                    
                } catch (IllegalAccessException e) {
                    String msg = "Unexpected IllegalAccessException:"+e;
                    logger.warn(msg);
                    throw new SAXException(msg);
                }
            }
        
            logger.debug("Finished w/: " + curTag);
        }

        public void characters(char ch[], int start, int length) {
            if(needPropVal) {
                propVal = XMLUtil.fromXMLUnicodeString(new String(ch, start, length)).trim();
                if (propVal!=null) argList.add(propVal);
                logger.debug("got propVal:"+propVal);
                needPropVal = false;
            } else {
    //            logger.debug("[characters] " + new String(ch, start, length));
            }    
        }

        public void ignorableWhitespace(char ch[], int start, int length) {
    //        logger.debug("[whitespace] " + new String(ch, start, length));
        }

        public void warning(SAXParseException ex) {
            System.err.println("[Warning] " + getLocationString(ex) + ": " + ex.getMessage());
        }

        public void error(SAXParseException ex) {
            System.err.println("[Error] " + getLocationString(ex) + ": " + ex.getMessage());
        }

        public void fatalError(SAXParseException ex) throws SAXException {
            System.err.println("[Fatal Error] " + getLocationString(ex) + ": " + ex.getMessage());
        }

        private String getLocationString(SAXParseException ex) {
            StringBuffer str = new StringBuffer();
            String systemId = ex.getSystemId();
            if(systemId!=null) {
                int index = systemId.lastIndexOf(47);
                if(index!=-1) systemId = systemId.substring(index + 1);
                str.append(systemId);
            }
            str.append(':');
            str.append(ex.getLineNumber());
            str.append(':');
            str.append(ex.getColumnNumber());
            return str.toString();
        }

    /*
        public boolean setMethod(Object target, Method m, Class paramType, String propVal) throws InvocationTargetException, IllegalAccessException {
            if (paramType.equals(String.class)) {
                m.invoke(target, new Object[] {propVal});
                return true;
            } else if (paramType.equals(boolean.class) || paramType.equals(Boolean.class)) {
                m.invoke(target, new Object[] {new Boolean(propVal)});
                return true;
            } else if (paramType.equals(byte.class) || paramType.equals(Byte.class)) {
                m.invoke(target, new Object[] {new Byte(propVal)});
                return true;
            } else if (paramType.equals(char.class) || paramType.equals(Character.class)) {
                byte b[] = propVal.getBytes();
                m.invoke(target, new Object[] {new Character((char)b[0])});
                return true;
            } else if (paramType.equals(double.class) || paramType.equals(Double.class)) {
                m.invoke(target, new Object[] {new Double(propVal)});
                return true;
            } else if (paramType.equals(float.class) || paramType.equals(Float.class)) {
                m.invoke(target, new Object[] {new Float(propVal)});
                return true;
            } else if (paramType.equals(int.class) || paramType.equals(Integer.class)) {
                m.invoke(target, new Object[] {new Integer(propVal)});
                return true;
            } else if (paramType.equals(long.class) || paramType.equals(Long.class)) {
                m.invoke(target, new Object[] {new Long(propVal)});
                return true;
            } else if (paramType.equals(short.class) || paramType.equals(Short.class)) {
                m.invoke(target, new Object[] {new Short(propVal)});
                return true;
            } else {
                return false;
            }
        }

        public boolean setField(Object target, Field field, Class paramType, String propVal) throws IllegalAccessException {
            if (paramType.equals(String.class)) {
                field.set(target, propVal);
                return true;
            } else if (paramType.equals(boolean.class) || paramType.equals(Boolean.class)) {
                field.set(target, new Boolean(propVal));
                return true;
            } else if (paramType.equals(byte.class) || paramType.equals(Byte.class)) {
                field.set(target, new Byte(propVal));
                return true;
            } else if (paramType.equals(char.class) || paramType.equals(Character.class)) {
                byte b[] = propVal.getBytes();
                field.set(target, new Character((char)b[0]));
                return true;
            } else if (paramType.equals(double.class) || paramType.equals(Double.class)) {
                field.set(target, new Double(propVal));
                return true;
            } else if (paramType.equals(float.class) || paramType.equals(Float.class)) {
                field.set(target, new Float(propVal));
                return true;
            } else if (paramType.equals(int.class) || paramType.equals(Integer.class)) {
                field.set(target, new Integer(propVal));
                return true;
            } else if (paramType.equals(long.class) || paramType.equals(Long.class)) {
                field.set(target, new Long(propVal));
                return true;
            } else if (paramType.equals(short.class) || paramType.equals(Short.class)) {
                field.set(target, new Short(propVal));
                return true;
            } else {
                return false;
            }
        }
    */
        protected Object resolve(Object id) {
            if (id==null) return null;
            if (id.equals("null")) return null;
            if (objMap.containsKey(id)) {
                return objMap.get(id);
            } else {
                return id.toString();
            }
        }
    }
    

    //--------------- HttpServlet --------------------------------
    /**
     * This class extends HttpServlet primarily for convenience, so that you easily
     * use it to set up a servlet environment. The servlet methods that handle requests
     * do not do anything; it all happens in the config.
     */
    public void init() throws ServletException {
        logger.info("Attempting to setup default ObjectRepository (HTTP interface)");
        String descriptor = getInitParameter(ASSEMBLY_DESCRIPTOR);
        String parser = getInitParameter(SAX_PARSER);
        this.assemble(null, this, descriptor, parser);
        logger.info("Assembly complete!");
    }
    

    public static void main(String[] args) {
        //assemble into the default ObjectRepository (you may need to put the 
        //xml file in the classes directory in order to find it if you are 
        //running this standalone)
        
        logger.info("Attempting to setup default ObjectRepository (Application interface)");
        new ObjectRepositoryAssembler().assemble("../WEB-INF/object-repository.xml");
        logger.info("Assembly complete!");

        //check the prop value
        System.out.println("TEST_STRING:"+TEST_STRING);
        System.out.println("TEST_INT:"+TEST_INT);
        System.out.println("TEST_BOOLEAN:"+TEST_BOOLEAN);

        //check and make sure it's there
        ObjectRepository or = ObjectRepository.getGlobalRepository();
        List keys = or.getStateKeys();
        if (keys!=null) {
            Iterator it = keys.iterator();
            while (it.hasNext()) {
                Object key = it.next();
                Object val = or.getState(key);
                System.out.println("Key:"+key+" Val:"+val);
            }
        }
    }
    
}
