/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PHashMap.java,v 1.4 2002/02/03 00:25:51 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.io.*;
import java.util.*;

/**
 * <p>This class extends AbstractPData (which provides the
 * parental/statemap functionality) and delegates most of 
 * the Map functionality back to an underlying HashMap
 */
public class PHashMap extends AbstractPData implements PMap {

    private Map map = new HashMap();


    //--------------- PHashMap -----------------------------------
    /**
     * Set the underlying store (you only really need to use
     * this method if you want to store the data in something 
     * other than a HashMap, which is the default)
     *
     * @param map the Map structure to be used as the underlying 
     *        store
     */
    public void setStore(Map imap) {
        map = imap;
    }
    

    //--------------- PMap ---------------------------------------
    /**
     * Removes all mappings from this map (optional operation). 
     */
    public void clear() {
        //we need to start by clearing parents for any PData values in the list
        Iterator it = map.values().iterator();
        while (it.hasNext()) {
            Object el = it.next();
            if (el!=null && el instanceof PData) {
                PData pdata = (PData) el;
                if (pdata.isInheritParents()) pdata.setParent(null);
            }
        }
    
        //now clear the list
        map.clear();
    }

    /**
     * Returns true if this map contains a mapping for the specified key. 
     */
    public boolean containsKey(Object key) {
        return map.containsKey(key);
    }

    /**
     * Returns true if this map maps one or more keys to the specified value. 
     */
    public boolean containsValue(Object value) {
        return map.containsValue(value);
    }

    /**
     * Returns a set view of the mappings contained in this map. 
     */
    public Set entrySet() {
        return map.entrySet();
    }

    /**
     * Returns the value to which this map maps the specified key. 
     */
    public Object get(Object key) {
        return map.get(key);
    }

    /**
     * Returns true if this map contains no key-value mappings. 
     */
    public boolean isEmpty() {
        return map.isEmpty();
    }

    /**
     * Returns a set view of the keys contained in this map. 
     */
    public Set keySet() {
        return map.keySet();
    }

    /**
     * Associates the specified value with the specified key in this map (optional operation). 
     */
    public Object put(Object key, Object value) {
    
        //this check is to ensure that the parental relationship is automatically
        //cleaned up from the item currently backing the specified key. The idea here
        //is that if you're removing an element (which you are effectively doing
        //via a put) then that element should no longer point to this object as its 
        //parent.
        Object curEl = map.get(key);
        if (curEl!=null && curEl instanceof PData) {
            PData pdata = (PData) curEl;
            if (pdata.isInheritParents()) pdata.setParent(null);
        }

        //this check is used to ensure the parental hierarchy is automatically
        //maintained. If you add an element to this list and that element implements
        //PData and that element has inheritParents=true, then this list should
        //automatically automatically become that objects parent
        if (value!=null && value instanceof PData) {
            PData pdata = (PData) value;
            if (pdata.isInheritParents()) pdata.setParent(this);
        }
    
        //put the value in the map
        return map.put(key, value);
    } 

    /**
     * Copies all of the mappings from the specified map to this map (optional operation). 
     */
    public void putAll(Map imap) {
    
        //iterate through all the key/values in the map...if the key already
        //exists in the current map it will be replacing data, so we need to 
        //clear the parental value on the existing data. Otherwise, treat it
        //like a straight "set" and just update the parental value on the incoming
        //data
        Iterator it = imap.keySet().iterator();
        while (it.hasNext()) {
            Object key = it.next();
            
            //see if there is an existing value for this key
            Object ovalue = map.get(key);
            if (ovalue!=null && ovalue instanceof PData) {
                PData pdata = (PData) ovalue;
                if (pdata.isInheritParents()) pdata.setParent(null);
            }
            
            //now look at the new value for the key
            Object nvalue = imap.get(key);
            if (nvalue!=null && nvalue instanceof PData) {
                PData pdata = (PData) nvalue;
                if (pdata.isInheritParents()) pdata.setParent(this);
            }
        }
    
        //put the map into our map
        map.putAll(imap);
    }

    /**
     * Removes the mapping for this key from this map if present (optional operation). 
     */
    public Object remove(Object key) {
        //see if there is an existing value for this key
        Object value = map.get(key);
        if (value!=null && value instanceof PData) {
            PData pdata = (PData) value;
            if (pdata.isInheritParents()) pdata.setParent(null);
        }
            
        //remove the key    
        return map.remove(key);
    }

    /**
     * Returns the number of key-value mappings in this map. 
     */
    public int size() {
        return map.size();
    }

    /**
     * Returns a collection view of the values contained in this map. 
     */
    public Collection values() {
        return map.values();
    }


    //--------------- Cloneable ----------------------------------
    /**
     * Returns a shallow copy of this <tt>ArrayList</tt> instance. (The
     * elements themselves are not copied.)
     *
     * @return  a clone of this <tt>ArrayList</tt> instance.
     */
    public Object clone() {
        try { 
            PHashMap phm = (PHashMap) super.clone();
            phm.map = new HashMap(map);            
            return phm;
        } catch (CloneNotSupportedException e) { 
            // this shouldn't happen, since we are Cloneable
            throw new InternalError();
        }
    }


    //--------------- Object -------------------------------------
    /**
     * Check object for equality. Will return true if the incoming 
     * object is a) non-null, b) the size of the underlying list 
     * structures is the same and c) the list containsAll() the
     * same elements
     *
     * @param obj the object we're comparing against
     * @return true if the objects are equal
     */
    public boolean equals(Object obj) {
        if (obj==null) return false;
        if (obj==this) return true;
        if (!(obj instanceof PMap)) return false;
        PMap pm = (PMap) obj;
        if (this.size()!=pm.size()) return false;
        Iterator it = this.keySet().iterator();
        while (it.hasNext()) {
            if (!pm.containsKey(it.next())) return false;
        }
        it = this.values().iterator();
        while (it.hasNext()) {
            if (!pm.containsValue(it.next())) return false;
        }
        return true;
    }

    /**
     * Returns the hash code value for this list. 
     */
    public int hashCode() {
        return map.hashCode();
    }


    //--------------- Utility Methods ----------------------------
    


}


