/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DOMUtil.java,v 1.7 2001/11/08 05:00:40 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;

import org.enhydra.barracuda.core.util.*;

/**
 * DOM related utility functions.
 */
public class DOMUtil {

    private static byte[] sep = System.getProperty("line.separator").getBytes();

    /**
     * Find the first text descendent node of an element.
     * This recursively looks more than one level to search
     * for text in font nodes, etc.
     *
     * @param node The starting node for the search.
     * @return The text node or null if not found.
     */
    public static Text findFirstText(Node node) {
        if (node instanceof Text) return (Text) node;
        for (Node child = node.getFirstChild(); child!=null; child = child.getNextSibling()) {
            Text text = findFirstText(child);
            if (text!=null) return text;
        }
        return null;
    }

    /**
     * Gets the first text descendent node of an element.
     * This recursively looks more than one level to search
     * for text in font nodes, etc. Throws a DOMException
     * if the Text object is not found.
     *
     * @param node The starting node for the search.
     * @return The text node or null if not found.
     * @throws DOMException if the Text object is not found
     */
    public static Text getFirstText(Node node) {
        Text text = findFirstText(node);
        if (text==null) {
            String msg = "No child text mode found for element";
            String id = getID(node);
            throw new DOMException((short) -1, msg+(id!=null ? "; id=\""+id+"\"" : ""));
        }
        return text;
    }

    /**
     * Automatically set text in a Node. Basically looks for 
     * the first child Text node; if it finds one, it sets the
     * text, if not, it creates a new one with the appropriate
     * text. Throws a DOMException if it's illegal to add a Text
     * child to the particular node.
     *
     * @param node the starting node for the search.
     * @param node the text to be added
     * @return the updated node
     * @throws DOMException if the Text object is not found
     */
/*
    public static Node setTextInNode(Node node, String text) {
        Text textComp = DOMUtil.findFirstText((Element) node);
        if (textComp==null) {
            textComp = node.getOwnerDocument().createTextNode(text);
            node.appendChild(textComp);
        } else {
            textComp.setData(""+text);
        }
        return node;
    }
*/


    /**
     * Automatically set text in a Node. Basically we find the first
     * Text node beneath the current node and replace it with a
     * CDATASection for the incoming text. All other Text nodes are
     * removed. Throws a DOMException if it's illegal to add a Text
     * child to the particular node.
     *
     * @param node the starting node for the search.
     * @param node the text to be set
     * @return the updated node
     * @throws DOMException if the Text object is not found
     */
    public static Node setTextInNode(Node node, String text, boolean allowMarkupInText) {
        //start by setting the value in the first text node we find with a comment
        Comment comment = node.getOwnerDocument().createComment("");
        Node newNode = null;
        
        //csc_092701.1 - support both encoded/unencoded text
        if (allowMarkupInText) newNode = node.getOwnerDocument().createCDATASection(text);
        else newNode = node.getOwnerDocument().createTextNode(text);
//System.out.println ("newNode: "+newNode);
        
        Text textComp = DOMUtil.findFirstText((Element) node);
//System.out.println ("textComp:"+textComp);        
        if (textComp==null) {
            node.appendChild(comment);
        } else {
            Node parent = textComp.getParentNode();
            parent.replaceChild(comment, textComp);
        }
        
        //now remove all the rest of the text nodes
        removeAllTextNodes(node);        

        //now replace the comment with the newNode
        Node parent = comment.getParentNode();
        parent.replaceChild(newNode, comment);
//System.out.println ("parent:  "+parent);        
//System.out.println ("result:  "+DOMUtil.findFirstText((Element) parent));        
//DOMUtil.printStackTrace(parent.getOwnerDocument().getDocumentElement());
        return node;
    }
    
    /**
     * Remove all text nodes below this node
     *
     * @param node The starting node for the search.
     */
    public static void removeAllTextNodes(Node node) {
        if (node==null) return;
        if (!node.hasChildNodes()) return;
        NodeList nl = node.getChildNodes();
        for (int i=nl.getLength()-1; i>=0; i--) {
            Node n = (Node) nl.item(i);        
            if (n instanceof Text) node.removeChild(n);
            else removeAllTextNodes(n);
        }
    }
    
    /**
     * Given a Node name, return the "id" attribute if it exists.
     * If it does not exist, return null instead. This is basically
     * just a convenience method to cast the node to element and 
     * return the id from that.
     *
     * @param node the node name in question
     * @return the id value for the given node, if it exists. null if 
     *        doesn't
     */
    public static String getID(Node node) {
        return getID(node, null);
    }

    /**
     * Given a Node name, return the "id" attribute if it exists.
     * If it does not exist, return nullResponse instead. This is basically
     * just a convenience method to cast the node to element and 
     * return the id from that.
     *
     * @param node the node name in question
     * @param nullResponse the response to be returned if the id attribute
     *        does not exist
     * @return the id value for the given node, if it exists. null if 
     *        doesn't
     */
    public static String getID(Node node, String nullResponse) {
        String nodeName = nullResponse;
        if (node instanceof Element) {
            nodeName = ((Element) node).getAttribute("id");
        }
        return nodeName;
    }

    /**
     * <p>utility method to recursively print the stack trace for a DOM Node</p>
     */
    public static void printStackTrace(Node node) {
        printStackTrace(node, 0, System.out);
    }
    
    /**
     * <p>utility method to recursively print the stack trace for a DOM Node</p>
     *
     * <p>Bounds: <br>
     * If depth < 0, the method returns immediately</p>
     *
     * @param int inset depth at which to start printing
     * @param out OutputStream to print to
     */
    public static void printStackTrace(Node node, int depth, OutputStream out) {
        if (depth<0) depth = 0;
        if (depth>25) depth = 25;
        String spaces = "                                                                              ";
        String inset = spaces.substring(0,depth*3);
//        String className = node.getClass().getName();
//        if (className.startsWith("org.w3c.")) className = Classes.getShortClassName(node.getClass());
//        print (out, inset+className+"@"+Integer.toHexString(node.hashCode()));            
        print (out, inset+node.getClass().getName()+"@"+Integer.toHexString(node.hashCode()));            
        StringBuffer sb = new StringBuffer(200);
        String sep = "";
        Iterator it = Classes.getAllInterfaces(node).iterator();
        while (it.hasNext()) {
            sb.append(sep+Classes.getShortClassName((Class) it.next()));
            sep = ", ";
        }
        print (out, inset+"   implements: {"+sb.toString()+"}");
        print (out, inset+"   name:"+node.getNodeName());
        print (out, inset+"   attr:"+(node.hasAttributes() ? "" : " (n/a)"));
        if (node.hasAttributes()) {
            NamedNodeMap nnm = node.getAttributes();
            for (int i=0,max=nnm.getLength(); i<max; i++) {
                Attr attr = (Attr) nnm.item(i);
                print (out, inset+"      "+attr.getName()+":"+attr.getValue());
            }        
            print (out, inset+"   /end attr");
        }
        print (out, inset+"   children:"+(node.hasChildNodes() ? "" : " (n/a)"));
        if (node.hasChildNodes()) {
            NodeList nl = node.getChildNodes();
            for (int i=0,max=nl.getLength(); i<max; i++) {
                printStackTrace(nl.item(i), depth+2, out);
            }        
            print (out, inset+"   /end children");
        }
        print (out, inset+"/end @" + Integer.toHexString(node.hashCode()));
        if (out!=null) try {out.flush();} catch (IOException ioe) {}
    }
    
    protected static void print(OutputStream out, String s) {
        if (out!=null) try {
            out.write(s.getBytes());
            out.write(sep);
        } catch (IOException ioe) {}
    }



}
