/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Encodings.java,v 1.5 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.dom.io;

import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;

// FIXME: The information here is limited, need to find a canonical
// source of names and encoding sizes, etc.  Create an XML file describing
// it.

/**
 * Class that manages information about encodings.
 */
public final class Encodings {
    /**
     * Some standard character set constants. Using the standard names.
     */
    public static final String ISO_8859_1 = "ISO_8859-1";
    public static final String US_ASCII = "US-ASCII";

    /**
     * Information about an encoding.
     */
    private class Encoding {
        /** Encoding name */
        private final String fName;

        /** Size of characters, in bits */
        private final int fCharSize;

        /** MIME prefered name, null if not specified. */
        private final String fMimePreferred;

        /** List of aliases, zero length if none */
        private final String[] fAliases;
        
        /** Constructor */
        public Encoding(String name,
                        int charSize,
                        String mimePreferred,
                        String[] aliases) {
            fName = name;
            fCharSize = charSize;
            fMimePreferred = mimePreferred;
            fAliases = aliases;
        }

        /** Get the name */
        public String getName() {
            return fName;
        }

        /** Get the character size */
        public int getCharSize() {
            return fCharSize;
        }

        /** Get the MIME preferred name or null if unspecified */
        public String getMIMEPreferred() {
            return fMimePreferred;
        }
        
        /** Get the aliases */
        public String[] getAliases() {
            return fAliases;
        }

        /** Get string representation of object */
        public String toString() {
            StringBuffer buf = new StringBuffer(256); // larger than default
            buf.append(fName);
            buf.append(": ");
            buf.append(fCharSize);
            buf.append(' ');
            buf.append(fMimePreferred);
            for (int i = 1; i < fAliases.length; i++) {
                buf.append(' ');
                buf.append(fAliases[i]);
            }
            return buf.toString();
        }
    }

    /**
     * Resource name of file containing character set information.
     */
//csc    private final String CHAR_SET_TABLE
//csc        = "org/enhydra/xml/io/character-sets.tbl";
    private final String CHAR_SET_TABLE
        = "character-sets.tbl";        //csc

    /**
     * String use for null encoding in file.
     */
    private final String NULL_STR = "null";

    /**
     * Singleton instance.
     */
    private static Encodings fSingleton;

    /**
     * Singleton empty list of aliases.
     */
    private static final String[] fEmptyAliases = new String[0];

    /**
     * Table of encodings.  The table is keyed by upper-cased encoding names
     * and aliases, containing references to Encoding objects.
     */
    private final HashMap fEncodings = new HashMap();

    /**
     * Generate an error about parsing an entry
     */
    private void parseError(String msg,
                            String line) {
        throw new XMLIOError(msg + "; parsing line in " + CHAR_SET_TABLE
                             + "\"" + line + "\"");
    }

    /**
     * Parse the next entry from the file
     */
    private Encoding parseCharSetEntry(String line) {
        StringTokenizer tokens = new StringTokenizer(line);
        int numTokens = tokens.countTokens();
        if (numTokens < 3) {
            parseError("must have at least 3 entries", line);
        }

        String name = tokens.nextToken();
        int charSetSize = 0;
        try {
            charSetSize = Integer.parseInt(tokens.nextToken());
        } catch (NumberFormatException nfe) {
            parseError("invalid character set size", line);
        }
        String mimePreferred = tokens.nextToken();;
        if (mimePreferred.equals(NULL_STR)) {
            mimePreferred = null;
        }
        String[] aliases = fEmptyAliases;

        if (numTokens > 3) {
            aliases = new String[numTokens - 3];
            int idx = 0;
            while (tokens.hasMoreTokens()) {
                aliases[idx++] = tokens.nextToken();
            }
        }

        return new Encoding(name, charSetSize, mimePreferred, aliases);
    }

    /**
     * Add an entry to the mapping table.
     */
    private void addEntry(Encoding entry) {
        fEncodings.put(entry.getName().toUpperCase().intern(), entry);
        
        String[] aliases = entry.getAliases();
        int len = (aliases == null) ? 0 : aliases.length;
        for (int idx = 0; idx < len; idx++) {
            fEncodings.put(aliases[idx].toUpperCase().intern(), entry);
        }
    }

    /**
     * Parse the character set file.
     */
    private void parseCharacterSetTable(BufferedReader in) throws IOException {
        String line;
        while ((line = in.readLine()) != null) {
            addEntry(parseCharSetEntry(line));
        }
    }

    /**
     * Parse the character set file.
     */
    private void parseCharacterSetTable() {
//csc_start
/*    
        ClassLoader loader = Encodings.class.getClassLoader();
        if (loader == null) {
            // Works around a JDeveloper JVM bug.
            loader = ClassLoader.getSystemClassLoader();
        }
        InputStream inStream = loader.getResourceAsStream(CHAR_SET_TABLE);
*/        
        InputStream inStream = this.getClass().getResourceAsStream(CHAR_SET_TABLE);
//csc_end
        
        if (inStream == null) {
            throw new XMLIOError("can't find \"" + CHAR_SET_TABLE + "\" on class path");
        }
        try {
            try {
                BufferedReader in = new BufferedReader(new InputStreamReader(inStream));
                parseCharacterSetTable(in);
            } finally {
                inStream.close();
            }
        } catch (IOException except) {
            throw new XMLIOError("error reading " + CHAR_SET_TABLE);
        }
    }

    /**
     * Disallow external instantiation.
     */
    private Encodings() {
        parseCharacterSetTable();
    }

    /**
     * Get the entry for an encoding name or null.
     */
    private Encoding getEntry(String encoding) {
        return (Encoding)fEncodings.get(encoding.toUpperCase());
    }

    /**
     * Determine if an encoding is a valid encoding.
     */
    public boolean isValid(String encoding) {
        return getEntry(encoding) != null;
    }
    
    /**
     * Get the encoding name, given any encoding or alias, or null
     * if unknown encoding.
     */
    public String getName(String encoding) {
        Encoding entry = getEntry(encoding);
        if (entry == null) {
            return null;
        } else {
            return entry.getName();
        }
    }

    /**
     * Get the preferred MIME encoding, or null if unknown encoding.
     */
    public String getMIMEPreferred(String encoding) {
        Encoding entry = getEntry(encoding);
        if (entry == null) {
            return null;
        } else {
            return entry.getMIMEPreferred();
        }
    }

    /**
     * Get the encoding aliases, given any encoding or alias,
     * or null if unknown encoding.
     */
    public String[] getAliases(String encoding) {
        Encoding entry = getEntry(encoding);
        if (entry == null) {
            return null;
        } else {
            return entry.getAliases();
        }
    }

    /**
     * Get the maximum value of an unicode character in an encoding.
     * This checks for 7 and 8 bit encodings; everything else is considered
     * to require 16 bits.
     */
    public int getMaxCharacterValue(String encoding) {
        Encoding entry = getEntry(encoding);
        if (entry != null) {
            int charSize = entry.getCharSize();
            if (charSize == 7) {
                return 0x7e;
            }
            if (charSize == 8) {
                return 0xFF;
            }
        }
        return 0xFFFF;
    }

    /**
     * Determine if two encoding names represent the same encoding.
     * Handles nulls, with two nulls being considered the same encoding.
     */
    public boolean sameEncodings(String encoding1,
                                 String encoding2) {
        if ((encoding1 == null) && (encoding2 == null)) {
            return true;
        } else if ((encoding1 == null) || (encoding2 == null)) {
            return false;
        } else {
            return (getEntry(encoding1) == getEntry(encoding2));
        }
    }

    /**
     * Return string representation for debugging.
     */
    public String toString() {
        StringBuffer buf = new StringBuffer(4096); // larger than default
        Iterator values = fEncodings.values().iterator();
        while (values.hasNext()) {
            buf.append(values.next().toString());
            buf.append('\n');
        }
        return buf.toString();
    }

    /**
     * Get the singleton instance of this class.
     */
    public static Encodings getEncodings() {
        if (fSingleton == null) {
            fSingleton = new Encodings();
        }
        return fSingleton;
    }
}
