/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: OutputOptions.java,v 1.4 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.dom.io;

import java.util.Hashtable;
import org.enhydra.xml.xmlc.codegen.JavaCode;
import org.enhydra.xml.xmlc.codegen.JavaLang;

/**
 * Object that specifies how a HTML or XML file will be formatted.
 * <P>
 * Note: the pretty-printing options are not yet implemented by the
 * formatters.
 */
public final class OutputOptions {
    // N.B. Be sure to modify createCodeGenerator and toString if any fields
    // are added or default values changed.

    /**
     * Desired output format enumerated type.
     */
    public static class Format {
        /** Name of format */
        private final String fName;

        /** Constructor (package only) */
        Format(String name) {
            fName = name;
        }
        
        /** Get string name of format */
        public String toString() {
            return fName;
        }
    }

    /**
     * Constant indicating format should be determined automatically
     * from examining the document object.
     */
    public static final Format FORMAT_AUTO = new Format("FORMAT_AUTO");

    /**
     * Constant indicating HTML format.
     */
    public static final Format FORMAT_HTML = new Format("FORMAT_HTML");

    /**
     * Constant indicating XML format.
     */
    public static final Format FORMAT_XML = new Format("FORMAT_XML");

    /**
     * The output file format to use.
     */
    private Format fFormat = FORMAT_AUTO;

    /**
     * The encoding to use.
     */
    private String fEncoding = null;

    /**
     * Should the document be pretty-printed?
     */
    private boolean fPrettyPrinting = false;

    /**
     * Number of characters to indent.
     */
    private int fIndentSize = 4;

    /**
     * Is space preserved where not otherwise specified by
     * the document?
     */
    private boolean fPreserveSpace = true;

    /**
     * Don't write the XML header.
     */
    private boolean fOmitXMLHeader = false;

    /**
     * Don't write the DOCTYPE for either XML or HTML.
     */
    private boolean fOmitDocType = false;

    /**
     * Don't add the encoding to the XML header.
     */
    private boolean fOmitEncoding = false;

    /**
     * MS Internet Explorer 4.0 get very confused about keep-alive connections
     * if HTML SPAN tags have ID attributes.  Until we find another workaround,
     * this removes IDs from SPAN tags.
     */
    private boolean fDropHtmlSpanIds = false;

    /**
     * Controls if character entity references are included in attribute values.
     */
    private boolean fOmitAttributeCharEntityRefs = false;

    /**
     * Rewriter for URL attribute values.
     */
    private URLRewriter fURLRewriter;

    /**
     * The public id to use in the DOCTYPE.  Overrides the DocumentType.
     */
    private String fPublicId;

    /**
     * The system id to use in the DOCTYPE.  Overrides the DocumentType.
     */
    private String fSystemId;

    /**
     * Optional MIME for an output routine to use.  This is stored in this
     * object for use by output routines, DOMFormatters don't actually use it.
     */
    private String fMIMEType;

    /**
     * Is this object read-only?
     */
    private boolean fReadOnly;

    /**
     * Construct with default values.
     */
    public OutputOptions() {
    }

    /**
     * Copy constructor.  The read-only property is not copied, the
     * resulting object maybe modified.
     */
    public OutputOptions(OutputOptions src) {
        fFormat = src.fFormat;
        fEncoding = src.fEncoding;
        fPrettyPrinting = src.fPrettyPrinting;
        fIndentSize = src.fIndentSize;
        fPreserveSpace = src.fPreserveSpace;
        fOmitXMLHeader = src.fOmitXMLHeader;
        fOmitDocType = src.fOmitDocType;
        fOmitEncoding = src.fOmitEncoding;
        fDropHtmlSpanIds = src.fDropHtmlSpanIds;
        fOmitAttributeCharEntityRefs = src.fOmitAttributeCharEntityRefs;
        fURLRewriter = src.fURLRewriter;
        fPublicId = src.fPublicId;
        fSystemId = src.fSystemId;
        fMIMEType = src.fMIMEType;
    }

    /**
     * Validate that the object is writable.
     */
    private void readOnlyCheck() {
        if (fReadOnly) {
            throw new XMLIOError(OutputOptions.class.getName() + " object is marked as read-only");
        }
    }

    /**
     * Mark the object as read-only.  Once made read-only, it may never be modified.
     * If a modification is required, create a new object using the copy-constructor.
     * @see #OutputOptions(OutputOptions)
     */
    public void markReadOnly() {
        fReadOnly = true;
    }

    /**
     * Set the output format for the file.  Specifying an incorrect format
     * will result in an invalid document.  Default is FORMAT_AUTO,
     * which determines the format from the DOM.
     */
    public void setFormat(Format format) {
        readOnlyCheck();
        fFormat = format;
    }

    /**
     * Get the output format for the file.
     */
    public Format getFormat() {
        return fFormat;
    }

    /**
     * Get the encoding.
     * @return The encoding or null if not specified.
     */
    public String getEncoding() {
        return fEncoding;
    }

    /**
     * Set the encoding.
     *
     * @param encoding The new encoding, or null to clear.
     */
    public void setEncoding(String encoding) {
        readOnlyCheck();
        fEncoding = encoding;
    }

    /**
     * Get the MIME encoding.
     * @return The MIME-preferred name for the encoding or null if
     *  no encoding is specified.
     */
    public String getMIMEEncoding() {
        if (fEncoding != null) {
            String mimeEncoding = Encodings.getEncodings().getMIMEPreferred(fEncoding);
            if (mimeEncoding != null) {
                return mimeEncoding;
            }
        }
        return fEncoding;
    }

    /**
     * Get pretty-printing flag.
     * @return true if enable, false to disable.  The default is disabled.
     */
    public boolean getPrettyPrinting() {
        return fPrettyPrinting;
    }

    /**
     * Enable or disable pretty-printing.
     * @param enable true to enable, false to disable.
     */
    public void setPrettyPrinting(boolean enable) {
        readOnlyCheck();
        fPrettyPrinting = enable;
    }

    /**
     * Get indentation size.
     * @return Number of characters to indent at each level.
     */
    public int getIndentSize() {
        return fIndentSize;
    }

    /**
     * Set indentation size.
     * @param size Number of characters to indent at each level.
     */
    public void setIndentSize(int size) {
        readOnlyCheck();
        fIndentSize = size;
        if (fIndentSize < 0) {
            fIndentSize = 0;
        }
    }

    /**
     * Get the default space-preservation flag.
     * @return true if preserving space where not otherwise specified by
     *  the document, false otherwise.  The default is true.
     */
    public boolean getPreserveSpace() {
        return fPreserveSpace;
    }

    /**
     * Set the default space-preservation flag.
     * @param perserve true if preserving space where not otherwise specified
     *  by the document
     */
    public void setPreserveSpace(boolean preserve) {
        readOnlyCheck();
        fPreserveSpace = preserve;
    }

    /**
     * Get flag indicating if the XML header should be omitted.
     */
    public boolean getOmitXMLHeader() {
        return fOmitXMLHeader;
    }

    /**
     * Set flag indicating if the XML header should be omitted.
     */
    public void setOmitXMLHeader(boolean omit) {
        readOnlyCheck();
        fOmitXMLHeader = omit;
    }

    /**
     * Get flag indicating if the DOCTYPE should be omitted.
     */
    public boolean getOmitDocType() {
        return fOmitDocType;
    }

    /**
     * Set flag indicating if the DOCTYPE should be omitted.
     */
    public void setOmitDocType(boolean omit) {
        readOnlyCheck();
        fOmitDocType = omit;
    }

    /**
     * Get flag indicating if encoding should be omitted from the XML header.
     */
    public boolean getOmitEncoding() {
        return fOmitEncoding;
    }

    /**
     * Set flag indicating if encoding should be omitted from the XML header.
     * This is provided as a hack for WML.  Several devices need ASCII
     * encoding but can't handle the header.
     */
    public void setOmitEncoding(boolean omit) {
        readOnlyCheck();
        fOmitEncoding = omit;
    }

    /**
     * Get the drop HTML SPAN element ids flag.
     */
    public boolean getDropHtmlSpanIds() {
        return fDropHtmlSpanIds;
    }

    /**
     * Set the drop HTML SPAN element ids flag.
     */
    public void setDropHtmlSpanIds(boolean drop) {
        readOnlyCheck();
        fDropHtmlSpanIds = drop;
    }

    /**
     * Get value of flag that enables or disables the use of character entity
     * references in attribute values.  By default, all standard character
     * entity references are used in attribute values. While this is legal in
     * HTML and XML, so HTML clients (for instance, one of the major browsers
     * didn't correctly expand the entity references in PARAM values passed to
     * applets).  If this flag is set, then standard character entity
     * references (such as &amp;amp;) will not be substituted.  Numeric
     * character entity references will still be substituted for quotes and
     * for characters that can't be represented in the encoding.  This option
     * is not recommended for XML.
     */
    public boolean getOmitAttributeCharEntityRefs() {
        return fOmitAttributeCharEntityRefs;
    }

    /**
     * Set value of flag that enables or disables the use of character entity
     * references in attribute values.
     * @see #getOmitAttributeCharEntityRefs
     */
    public void setOmitAttributeCharEntityRefs(boolean value) {
        fOmitAttributeCharEntityRefs = value;
    }

    /**
     * Set the URLRewriter that all URL attributes will be passed through.
     * Documents must implement DocumentInfo for the URL rewriter to
     * work.
     * <BR>
     * Note: URL rewriting curret only works when the XMLC document
     * object is passed to the formatter.  Passing the contained document or
     * any other node results in no URL rewriting.
     *
     * @param urlRewriter The URLRewriter object, or null to disassociate
     *  any URL rewriter.
     * @see DocumentInfo
     */
    public void setURLRewriter(URLRewriter urlRewriter) {
        readOnlyCheck();
        fURLRewriter = urlRewriter;
    }

    /**
     * Get the URLRewriter.
     * @return The URLRewriter object. 
     * @see DocumentInfo
     */
    public URLRewriter getURLRewriter() {
        return fURLRewriter;
    }

    /**
     * Get the public id to use in the DOCUMENT.  This overrides the
     * default value determined from the DocumentType.
     *
     * @return the public id
     */
    public String getPublicId() {
        return fPublicId;
    }

    /**
     * Set the public id to use in the DOCUMENT.  This overrides the
     * default value determined from the DocumentType.
     *
     * @return the public id
     */
    public void setPublicId(String publicId) {
        readOnlyCheck();
        fPublicId = publicId;
    }

    /**
     * Get the system id to use in the DOCUMENT.  This overrides the
     * default value determined from the DocumentType.
     *
     * @return the system id
     */
    public String getSystemId() {
        return fSystemId;
    }

    /**
     * Set the system id to use in the DOCUMENT.  This overrides the
     * default value determined from the DocumentType.
     *
     * @return the system id
     */
    public void setSystemId(String systemId) {
        readOnlyCheck();
        fSystemId = systemId;
    }

    /**
     * Get the MIME for an output routine to use.
     *
     * @return The overriding MIME typen ull if one was not specified.
     */
    public String getMIMEType() {
        return fMIMEType;
    }

    /**
     * Set the MIME for an output routine to use.  This is stored in this
     * object for use by output routines, DOMFormatters don't actually use it.
     * It is normally used to override the default MIME type that would be
     * stored in the XMLObject.
     */
    public void setMIMEType(String mimeType) {
        readOnlyCheck();
        fMIMEType = mimeType;
    }

    /**
     * Generate code to set a String property.
     */
    private void genSet(String varName,
                        JavaCode code,
                        String setMethod,
                        String value) {
        code.addln(varName + "." + setMethod + "("
                   + JavaLang.createStringConst(value) + ");");
    }

    /**
     * Generate code to set a boolean property.
     */
    private void genSet(String varName,
                        JavaCode code,
                        String setMethod,
                        boolean value) {
        code.addln(varName + "." + setMethod + "("
                   + (value ? "true" : "false") + ");");
    }

    /**
     * Generate code to set an int property.
     */
    private void genSet(String varName,
                        JavaCode code,
                        String setMethod,
                        int value) {
        code.addln(varName + "." + setMethod + "("
                   + Integer.toString(value) + ");");
    }

    /**
     * Generate code to create an object with the same attributes as this
     * object.
     * <P>
     * The following attributes are not generated in the new code:
     * <UL>
     * <LI> readOnly - Set to the value of the makeReadOnly parameter.
     * <LI> urlRewriter
     * </UL>
     *
     * @param varName Variable or field name of variable to store
     *  the object in.  It must already be declared.
     * @param makeReadOnly Should the created object be made read-only?
     * @param code Add generated code to this object.
     */
    public void createCodeGenerator(String varName,
                                    boolean makeReadOnly,
                                    JavaCode code) {
        //FIXME: Change to only generate code if not default
        code.addln(varName + " = new " + OutputOptions.class.getName() + "();");
        code.addln(varName + ".setFormat("
                   + OutputOptions.class.getName()
                   + "." + fFormat.toString() + ");");

        // Java encoding set by setting XML encoding
        genSet(varName, code, "setEncoding", fEncoding);
        genSet(varName, code, "setPrettyPrinting", fPrettyPrinting);
        genSet(varName, code, "setIndentSize", fIndentSize);
        genSet(varName, code, "setPreserveSpace", fPreserveSpace);
        genSet(varName, code, "setOmitXMLHeader", fOmitXMLHeader);
        genSet(varName, code, "setOmitDocType", fOmitDocType);
        genSet(varName, code, "setOmitEncoding", fOmitEncoding);
        genSet(varName, code, "setDropHtmlSpanIds", fDropHtmlSpanIds);
        genSet(varName, code, "setOmitAttributeCharEntityRefs", fOmitAttributeCharEntityRefs);
        genSet(varName, code, "setPublicId", fPublicId);
        genSet(varName, code, "setSystemId", fSystemId);
        genSet(varName, code, "setMIMEType", fMIMEType);
        if (makeReadOnly) {
            code.addln(varName + ".markReadOnly();");
        }
    }

    /**
     * Convert to a string for debugging.
     */
    public String toString() {
        return "readOnly=" + fReadOnly
            + ", format=" + fFormat
            + ", encoding=" + fEncoding
            + ", prettyPrinting=" + fPrettyPrinting
            + ", indentSize=" + fIndentSize
            + ", preserveSpace=" + fPreserveSpace
            + ", omitXMLHeader=" + fOmitXMLHeader
            + ", omitDocType=" + fOmitDocType
            + ", omitEncoding=" + fOmitEncoding
            + ", dropHtmlSpanIds=" + fDropHtmlSpanIds
            + ", omitAttributeCharEntityRefs=" + fOmitAttributeCharEntityRefs
            + ", urlRewriter=" + fURLRewriter
            + ", publicId=" + fPublicId
            + ", systemId=" + fSystemId
            + ", mimeType=" + fMIMEType;
    }

    /**
     * Get the encoding.
     * @deprecated Use <tt>getEncoding()</tt> or <tt>getMIMEEncoding()</tt>.
     * @see #getEncoding
     * @see #getMIMEEncoding
     */
    public String getJavaEncoding() {
        return getEncoding();
    }

    /**
     * Set the encoding.
     * @deprecated Use <tt>setEncoding()</tt>.
     * @see #setEncoding
     */
    public void setJavaEncoding(String newJavaEncoding) {
        setEncoding(newJavaEncoding);
    }

    /**
     * Get the MIME-preferred encoding.
     * @deprecated Use <tt>getMIMEEncoding()</tt>.
     * @see #getMIMEEncoding
     */
    public String getXmlEncoding() {
        return getMIMEEncoding();
    }

    /**
     * Set the encoding.
     * @deprecated Use <tt>setEncoding()</tt>.
     */
    public void setXmlEncoding(String newXmlEncoding) {
        setEncoding(newXmlEncoding);
    }

}
