/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *  @author     christianc@lutris.com
 *
 * $Id: CopyAndReplace.java,v 1.6 2002/07/12 18:45:23 cryd0221 Exp $
 */
package org.enhydra.barracuda.taskdefs;

import java.util.*;
import java.io.*;
import org.apache.tools.ant.*;
import org.apache.tools.ant.taskdefs.*;
import org.apache.tools.ant.types.*;

/**
 * <p>A consolidated copy and replace task that extends the basic Ant Copy
 * taskdef.
 *
 * <p>In addition, once files have been copied, a replace function is also
 * run on them. This replace will only affect the files that actually
 * get copied; it will not affect any other files in the directory.
 *
 * <p>The replace occurs using a simply properties file, rather than 
 * specifying replace parameters via the xml. The format of this file
 * looks something like this:
 *
 * <ul>
 *   <li>token="some target text" value="some replacement text"</li>
 *   <li>token='some target text called "blah"' value='some replacement text named "blech"'</li>
 *   <li>token=~some target text with " and '~ value=^some replacement text with ' and "^</li>
 * </ul>
 *
 * <p>As you can see, the format is flexible. Tokens are identified with "token=" and 
 * values are identified by "value=". The actual token/value delimiters are 
 * taken to be the first character following the = sign. This could be a double
 * quote, single quote, or any other character (ie. if the text you wish to 
 * replace contains both double and single quotes, you might want to use a ~
 * delimiter or something like that).
 *
 * <p>Note also that this taskdef support SSI replacement, which can be quite useful.
 * By default, the copied files are NOT processed for SSIs; if you want to use this,
 * turn is on using the 'ssi=true' attribute. SSI replacement occurs BEFORE the token
 * replacement (ensuring that the text contained in the SSI gets processed for tokens
 * as well).
 *
 * @author Christian Cryder <a href="mailto:christianc@enhydra.org">christianc@enhydra.org</a>
 */
public class CopyAndReplace extends Copy {

    protected File mappingsFile = null; //the mappings file 
    protected boolean ssi = false;      //process for ssi's         //csc_111601.1 - added

    /**
     * Sets the mappings file.
     */
    public void setMappings(File mappingsFile) {
        this.mappingsFile = mappingsFile;
    }

    //csc_111601.1 - added
    /**
     * Process files copied for ssi
     * @param ssi set "true" to process copied files for ssi
     */
    public void setSsi(BooleanAttribute issi) {
        ssi = (issi.getValue().equals("yes") || issi.getValue().equals("true"));
    }

    /**
     * <p>Actually does the file (and possibly empty directory) copies.
     * This is a good method for subclasses to override. 
     *
     * <p>Note that all the copy functionality occurs by simply deferring
     * to the superclass implementation. The replace functionality follows:
     *
     * <ul>
     *    <li><p>first we make sure there is a mappings file</li>
     *    <li><p>next we parse it to determine all token/value mapings</li>
     *    <li><p>finally we iterate through the list of files that actually
     *          got copied and we create Replace task for each of them. This
     *          task contains all the various token/value mappings, and
     *          gets executed for each file, effectively making all the 
     *          necessary text substitutions</li>
     * <ul>
     */
    protected void doFileOperations() {
        //start by allowing the basic copy to occur
        super.doFileOperations();
        
        //if the mappings file is null or doesn't exist just return
        if (mappingsFile==null) return;
        List mappings = new ArrayList();
        if (!mappingsFile.exists()) {
            log("Unable to find mappings file: "+mappingsFile+" ... files copied but no text replace occurred", Project.MSG_ERR);
            return;            
        }

        //start by reading the mappings file
        List tokenList = new ArrayList();
        List valueList = new ArrayList();
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(mappingsFile)));
            while (true) {
                String s = br.readLine();
                if (s==null) break;
                String token = null;
                String value = null;

                //get the token                
                int spos = s.indexOf("token=");
                if (spos<0 || spos+6>=s.length()) continue;
                spos+=6;
                String delim = s.substring(spos,spos+1);
                spos+=1;
                int epos = s.indexOf(delim, spos+1);
                if (epos<0 || epos+1>=s.length()) continue;
                token = s.substring(spos, epos);
                if (token==null) continue;

                //get the value
                spos = s.indexOf("value=", epos+1);
                if (spos<0 || spos+6>=s.length()) continue;
                spos+=6;
                delim = s.substring(spos,spos+1);
                spos+=1;
                epos = s.indexOf(delim, spos+1);
                if (epos<0) continue;
                value = s.substring(spos, epos);
                if (value==null) continue;
                
                tokenList.add(token);
                valueList.add(value);
            }
        } catch (IOException e) {
            System.out.println ("Error reading file "+mappingsFile+":"+e);
            return;
        }

        //csc_111601.1_start
        //support ssi
        if (ssi) {
            if (fileCopyMap.size()>0) log("Processing copied files for server side includes");

            Enumeration enum = fileCopyMap.keys();
            while (enum.hasMoreElements()) {
                //figure out the target file
                String fromFile = (String) enum.nextElement();
                String toFile = (String) fileCopyMap.get(fromFile);
                File targetFile = new File(toFile);
                File fromDir = new File(fromFile).getParentFile();

                //get the file (this will automatically resolve ssi's)
                byte[] dstbytes = getFile(targetFile, fromDir);
                if (dstbytes==null) continue;
                
                //now write the file back out
                try {
                    FileOutputStream fos = new FileOutputStream(targetFile);
                    fos.write(dstbytes);
                    fos.close();
                } catch (IOException e) {
                    System.out.println("Error writing file "+targetFile+":"+e);
                    continue;
                }
            }        
        }
        //csc_111601.1_end
        
        //now iterate through the file copy list
        if (fileCopyMap.size()>0) {
            log("Replacing text in copied files");

            Enumeration enum = fileCopyMap.keys();
            while (enum.hasMoreElements()) {
                //figure out the target file
                String fromFile = (String) enum.nextElement();
                String toFile = (String) fileCopyMap.get(fromFile);
                File targetFile = new File(toFile);

                //do the replace on each file (this is important: we only
                //want to do the replace on files we actually copied in)
                Replace replace = new Replace();
                replace.setProject(project);
                replace.setFile(targetFile);
                for (int i=0, max=tokenList.size(); i<max; i++) {
                    Replace.Replacefilter rf = replace.createReplacefilter();
                    rf.setToken((String) tokenList.get(i));
                    rf.setValue((String) valueList.get(i));
                }
                replace.execute();
            }        
        }
    }
    
    public static class BooleanAttribute extends EnumeratedAttribute {
        public String[] getValues() {
            return new String[] {"yes", "no", "true", "false"};
        }
    }
    

    //csc_111601.1_start
    private static String START_SSI = "<!--#include file=\"";
    private static String END_SSI = "\"-->";

    public byte[] getFile(File targetFile, File fromDir) {
        //adjust the target file
        if (!targetFile.exists() && fromDir!=null) {
            String targetFileName = targetFile.getName();
            targetFile = new File(fromDir, targetFileName);
        }
    
        //get the resource
        byte[] dstbytes = new byte[10240];
        try {
            //read in the file and store it in a byte array
            FileInputStream fis = new FileInputStream(targetFile);
            BufferedInputStream in = new BufferedInputStream(fis);
            int offset = 0;
            int len = 1024;
            byte[] inbytes = new byte[len];
            while (true) {
                int cnt = in.read(inbytes, 0, len);
                if (cnt==-1) break;
                if (offset+cnt>=dstbytes.length) {
                    byte[] newbytes = new byte[(int) (dstbytes.length+(len*5))];
                    System.arraycopy(dstbytes, 0, newbytes, 0, dstbytes.length);
                    dstbytes = newbytes;
                }
                System.arraycopy(inbytes, 0, dstbytes, offset, cnt);
                offset+=cnt;
            }
            in.close();
        
            //when we're done trim off the excess
            byte[] newbytes = new byte[offset];
            System.arraycopy(dstbytes, 0, newbytes, 0, offset);
            dstbytes = newbytes;
            
            //now create a new string from the result, and process it for ssi's
            //(this is not exactly the most efficient way of doing things, but it'll
            //work for now...if you don't like it, optimize it and submit it back 
            //to the group ;-)
            String result = new String(dstbytes);
            int spos = 0;
            int epos = 0;
            while (true) {
                spos = result.indexOf(START_SSI, epos);
                if (spos<0) break;
                epos = result.indexOf(END_SSI, spos);
                if (epos<spos) break;
                String ssiName = result.substring(spos+START_SSI.length(), epos);
                File ssiFile = null;
                if (ssiFile==null || !ssiFile.exists()) ssiFile = new File(targetFile.getParent(), ssiName);
                if (ssiFile==null || !ssiFile.exists()) ssiFile = new File(fromDir, ssiName);
                if (ssiFile==null || !ssiFile.exists()) ssiFile = new File(ssiName);
                if (ssiFile==null || !ssiFile.exists()) {
                    System.out.println ("Error locating SSI:"+ssiName);
                    continue;
                }
                byte[] ssiBytes = getFile(ssiFile, null);
                if (ssiBytes!=null) result = result.substring(0,spos)+new String(ssiBytes)+result.substring(epos+END_SSI.length());
            }
            dstbytes = result.getBytes();
            
        } catch (IOException e) {
            System.out.println ("Error reading file "+targetFile+":"+e);
            return null;
        }
        return dstbytes;
    }    
    //csc_111601.1_end
}
